@@ -1,5 +1,5 @@
 /*============================================================================
-  WCSLIB 7.4 - an implementation of the FITS WCS standard.
+  WCSLIB 7.7 - an implementation of the FITS WCS standard.
   Copyright (C) 1995-2021, Mark Calabretta
 
   This file is part of WCSLIB.
@@ -19,13 +19,13 @@
 
   Author: Mark Calabretta, Australia Telescope National Facility, CSIRO.
   http://www.atnf.csiro.au/people/Mark.Calabretta
-  $Id: lin.c,v 7.4 2021/01/31 02:24:51 mcalabre Exp $
+  $Id: lin.c,v 7.7 2021/07/12 06:36:49 mcalabre Exp $
 *===========================================================================*/
 
+#include <math.h>
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
-#include <math.h>
 
 #include "wcserr.h"
 #include "wcsprintf.h"
@@ -72,17 +72,13 @@ int lininit(int alloc, int naxis, struct linprm *lin, int ndpmax)
 {
   static const char *function = "lininit";
 
-  int i, j;
-  double *pc;
-  struct wcserr **err;
-
   if (lin == 0x0) return LINERR_NULL_POINTER;
 
   // Initialize error message handling.
   if (lin->flag == -1) {
     lin->err = 0x0;
   }
-  err = &(lin->err);
+  struct wcserr **err = &(lin->err);
   wcserr_clear(err);
 
 
@@ -204,14 +200,14 @@ int lininit(int alloc, int naxis, struct linprm *lin, int ndpmax)
 
 
   // CRPIXja defaults to 0.0.
-  for (j = 0; j < naxis; j++) {
+  for (int j = 0; j < naxis; j++) {
     lin->crpix[j] = 0.0;
   }
 
   // PCi_ja defaults to the unit matrix.
-  pc = lin->pc;
-  for (i = 0; i < naxis; i++) {
-    for (j = 0; j < naxis; j++) {
+  double *pc = lin->pc;
+  for (int i = 0; i < naxis; i++) {
+    for (int j = 0; j < naxis; j++) {
       if (j == i) {
         *pc = 1.0;
       } else {
@@ -222,7 +218,7 @@ int lininit(int alloc, int naxis, struct linprm *lin, int ndpmax)
   }
 
   // CDELTia defaults to 1.0.
-  for (i = 0; i < naxis; i++) {
+  for (int i = 0; i < naxis; i++) {
     lin->cdelt[i] = 1.0;
   }
 
@@ -245,11 +241,8 @@ int lindist(int sequence, struct linprm *lin, struct disprm *dis, int ndpmax)
 {
   static const char *function = "lindist";
 
-  int status;
-  struct wcserr **err;
-
   if (lin == 0x0) return LINERR_NULL_POINTER;
-  err = &(lin->err);
+  struct wcserr **err = &(lin->err);
 
   if (sequence == 1) {
     if (lin->m_dispre) {
@@ -277,7 +270,8 @@ int lindist(int sequence, struct linprm *lin, struct disprm *dis, int ndpmax)
   }
 
   if (dis) {
-    if ((status = disinit(1, lin->naxis, dis, ndpmax))) {
+    int status = disinit(1, lin->naxis, dis, ndpmax);
+    if (status) {
       return wcserr_set(LIN_ERRMSG(lin_diserr[status]));
     }
   }
@@ -292,42 +286,38 @@ int lincpy(int alloc, const struct linprm *linsrc, struct linprm *lindst)
 {
   static const char *function = "lincpy";
 
-  int i, j, naxis, status;
-  const double *srcp;
-  double *dstp;
-  struct wcserr **err;
-
   if (linsrc == 0x0) return LINERR_NULL_POINTER;
   if (lindst == 0x0) return LINERR_NULL_POINTER;
-  err = &(lindst->err);
+  struct wcserr **err = &(lindst->err);
 
-  naxis = linsrc->naxis;
+  int naxis = linsrc->naxis;
   if (naxis < 1) {
     return wcserr_set(WCSERR_SET(LINERR_MEMORY),
       "naxis must be positive (got %d)", naxis);
   }
 
-  if ((status = lininit(alloc, naxis, lindst, 0))) {
+  int status = lininit(alloc, naxis, lindst, 0);
+  if (status) {
     return status;
   }
 
-  srcp = linsrc->crpix;
-  dstp = lindst->crpix;
-  for (j = 0; j < naxis; j++) {
+  const double *srcp = linsrc->crpix;
+  double *dstp = lindst->crpix;
+  for (int j = 0; j < naxis; j++) {
     *(dstp++) = *(srcp++);
   }
 
   srcp = linsrc->pc;
   dstp = lindst->pc;
-  for (i = 0; i < naxis; i++) {
-    for (j = 0; j < naxis; j++) {
+  for (int i = 0; i < naxis; i++) {
+    for (int j = 0; j < naxis; j++) {
       *(dstp++) = *(srcp++);
     }
   }
 
   srcp = linsrc->cdelt;
   dstp = lindst->cdelt;
-  for (i = 0; i < naxis; i++) {
+  for (int i = 0; i < naxis; i++) {
     *(dstp++) = *(srcp++);
   }
 
@@ -442,11 +432,66 @@ int linfree(struct linprm *lin)
 
 //----------------------------------------------------------------------------
 
+int linsize(const struct linprm *lin, int sizes[2])
+
+{
+  if (lin == 0x0) {
+    sizes[0] = sizes[1] = 0;
+    return LINERR_SUCCESS;
+  }
+
+  // Base size, in bytes.
+  sizes[0] = sizeof(struct linprm);
+
+  // Total size of allocated memory, in bytes.
+  sizes[1] = 0;
+
+  int naxis = lin->naxis;
+
+  // linprm::crpix[].
+  sizes[1] += naxis * sizeof(double);
+
+  // linprm::pc[].
+  sizes[1] += naxis*naxis * sizeof(double);
+
+  // linprm::cdelt[].
+  sizes[1] += naxis * sizeof(double);
+
+  // linprm::dispre[].
+  int exsizes[2];
+  dissize(lin->dispre, exsizes);
+  sizes[1] += exsizes[0] + exsizes[1];
+
+  // linprm::disseq[].
+  dissize(lin->disseq, exsizes);
+  sizes[1] += exsizes[0] + exsizes[1];
+
+  // linprm::err[].
+  wcserr_size(lin->err, exsizes);
+  sizes[1] += exsizes[0] + exsizes[1];
+
+  // The remaining arrays are allocated unconditionally by linset().
+  if (lin->flag != LINSET) {
+    return LINERR_SUCCESS;
+  }
+
+  // linprm::piximg[].
+  sizes[1] += naxis*naxis * sizeof(double);
+
+  // linprm::imgpix[].
+  sizes[1] += naxis*naxis * sizeof(double);
+
+  // linprm::tmpcrd[].
+  sizes[1] += naxis * sizeof(double);
+
+  return LINERR_SUCCESS;
+}
+
+//----------------------------------------------------------------------------
+
 int linprt(const struct linprm *lin)
 
 {
-  int i, j, k;
-
   if (lin == 0x0) return LINERR_NULL_POINTER;
 
   if (lin->flag != LINSET) {
@@ -460,16 +505,16 @@ int linprt(const struct linprm *lin)
 
   WCSPRINTF_PTR("      crpix: ", lin->crpix, "\n");
   wcsprintf("            ");
-  for (j = 0; j < lin->naxis; j++) {
+  for (int j = 0; j < lin->naxis; j++) {
     wcsprintf("  %#- 11.5g", lin->crpix[j]);
   }
   wcsprintf("\n");
 
-  k = 0;
+  int k = 0;
   WCSPRINTF_PTR("         pc: ", lin->pc, "\n");
-  for (i = 0; i < lin->naxis; i++) {
+  for (int i = 0; i < lin->naxis; i++) {
     wcsprintf("    pc[%d][]:", i);
-    for (j = 0; j < lin->naxis; j++) {
+    for (int j = 0; j < lin->naxis; j++) {
       wcsprintf("  %#- 11.5g", lin->pc[k++]);
     }
     wcsprintf("\n");
@@ -477,7 +522,7 @@ int linprt(const struct linprm *lin)
 
   WCSPRINTF_PTR("      cdelt: ", lin->cdelt, "\n");
   wcsprintf("            ");
-  for (i = 0; i < lin->naxis; i++) {
+  for (int i = 0; i < lin->naxis; i++) {
     wcsprintf("  %#- 11.5g", lin->cdelt[i]);
   }
   wcsprintf("\n");
@@ -493,10 +538,10 @@ int linprt(const struct linprm *lin)
   if (lin->piximg == 0x0) {
     wcsprintf("     piximg: (nil)\n");
   } else {
-    k = 0;
-    for (i = 0; i < lin->naxis; i++) {
+    int k = 0;
+    for (int i = 0; i < lin->naxis; i++) {
       wcsprintf("piximg[%d][]:", i);
-      for (j = 0; j < lin->naxis; j++) {
+      for (int j = 0; j < lin->naxis; j++) {
         wcsprintf("  %#- 11.5g", lin->piximg[k++]);
       }
       wcsprintf("\n");
@@ -506,10 +551,10 @@ int linprt(const struct linprm *lin)
   if (lin->imgpix == 0x0) {
     wcsprintf("     imgpix: (nil)\n");
   } else {
-    k = 0;
-    for (i = 0; i < lin->naxis; i++) {
+    int k = 0;
+    for (int i = 0; i < lin->naxis; i++) {
       wcsprintf("imgpix[%d][]:", i);
-      for (j = 0; j < lin->naxis; j++) {
+      for (int j = 0; j < lin->naxis; j++) {
         wcsprintf("  %#- 11.5g", lin->imgpix[k++]);
       }
       wcsprintf("\n");
@@ -587,20 +632,16 @@ int linset(struct linprm *lin)
 {
   static const char *function = "linset";
 
-  int i, j, naxis, status;
-  double *pc, *piximg;
-  struct wcserr **err;
-
   if (lin == 0x0) return LINERR_NULL_POINTER;
-  err = &(lin->err);
+  struct wcserr **err = &(lin->err);
 
-  naxis = lin->naxis;
+  int naxis = lin->naxis;
 
   // Check for a unit matrix.
   lin->unity = 1;
-  pc = lin->pc;
-  for (i = 0; i < naxis; i++) {
-    for (j = 0; j < naxis; j++) {
+  double *pc = lin->pc;
+  for (int i = 0; i < naxis; i++) {
+    for (int j = 0; j < naxis; j++) {
       if (j == i) {
         if (*(pc++) != 1.0) {
           lin->unity = 0;
@@ -628,7 +669,7 @@ int linset(struct linprm *lin)
     lin->i_naxis = 0;
 
     // Check cdelt.
-    for (i = 0; i < naxis; i++) {
+    for (int i = 0; i < naxis; i++) {
       if (lin->cdelt[i] == 0.0) {
         return wcserr_set(LIN_ERRMSG(LINERR_SINGULAR_MTX));
       }
@@ -656,23 +697,24 @@ int linset(struct linprm *lin)
     }
 
     // Compute the pixel-to-image transformation matrix.
-    pc     = lin->pc;
-    piximg = lin->piximg;
-    for (i = 0; i < naxis; i++) {
+    pc = lin->pc;
+    double *piximg = lin->piximg;
+    for (int i = 0; i < naxis; i++) {
       if (lin->disseq == 0x0) {
         // No sequent distortions.  Incorporate cdelt into piximg.
-        for (j = 0; j < naxis; j++) {
+        for (int j = 0; j < naxis; j++) {
           *(piximg++) = lin->cdelt[i] * (*(pc++));
         }
       } else {
-        for (j = 0; j < naxis; j++) {
+        for (int j = 0; j < naxis; j++) {
           *(piximg++) = *(pc++);
         }
       }
     }
 
     // Compute the image-to-pixel transformation matrix.
-    if ((status = matinv(naxis, lin->piximg, lin->imgpix))) {
+    int status = matinv(naxis, lin->piximg, lin->imgpix);
+    if (status) {
       return wcserr_set(LIN_ERRMSG(status));
     }
   }
@@ -681,7 +723,8 @@ int linset(struct linprm *lin)
   // Set up the distortion functions.
   lin->affine = 1;
   if (lin->dispre) {
-    if ((status = disset(lin->dispre))) {
+    int status = disset(lin->dispre);
+    if (status) {
       return wcserr_set(LIN_ERRMSG(lin_diserr[status]));
     }
 
@@ -689,7 +732,8 @@ int linset(struct linprm *lin)
   }
 
   if (lin->disseq) {
-    if ((status = disset(lin->disseq))) {
+    int status = disset(lin->disseq);
+    if (status) {
       return wcserr_set(LIN_ERRMSG(lin_diserr[status]));
     }
 
@@ -724,33 +768,29 @@ int linp2x(
 {
   static const char *function = "linp2x";
 
-  int i, j, k, n, ndbl, nelemn, status;
-  double temp;
-  register const double *pix;
-  register double *img, *piximg, *tmp;
-  struct wcserr **err;
-
-
   // Initialize.
   if (lin == 0x0) return LINERR_NULL_POINTER;
-  err = &(lin->err);
+  struct wcserr **err = &(lin->err);
 
   if (lin->flag != LINSET) {
-    if ((status = linset(lin))) return status;
+    int status = linset(lin);
+    if (status) {
+      return status;
+    }
   }
 
-  n = lin->naxis;
+  int naxis = lin->naxis;
 
 
   // Convert pixel coordinates to intermediate world coordinates.
-  pix = pixcrd;
-  img = imgcrd;
+  register const double *pix = pixcrd;
+  register double *img = imgcrd;
 
   if (lin->simple) {
     // Handle the simplest and most common case with maximum efficiency.
-    nelemn = nelem - n;
-    for (k = 0; k < ncoord; k++) {
-      for (i = 0; i < n; i++) {
+    int nelemn = nelem - naxis;
+    for (int k = 0; k < ncoord; k++) {
+      for (int i = 0; i < naxis; i++) {
         *(img++) = lin->cdelt[i] * (*(pix++) - lin->crpix[i]);
       }
 
@@ -760,18 +800,18 @@ int linp2x(
 
   } else if (lin->affine) {
     // No distortions.
-    ndbl   = n * sizeof(double);
-    nelemn = nelem - n;
-    for (k = 0; k < ncoord; k++) {
+    int ndbl   = naxis * sizeof(double);
+    int nelemn = nelem - naxis;
+    for (int k = 0; k < ncoord; k++) {
       memset(img, 0, ndbl);
 
-      for (j = 0; j < n; j++) {
+      for (int j = 0; j < naxis; j++) {
         // cdelt will have been incorporated into piximg.
-        piximg = lin->piximg + j;
+        register double *piximg = lin->piximg + j;
 
         // Column-wise multiplication allows this to be cached.
-        temp = *(pix++) - lin->crpix[j];
-        for (i = 0; i < n; i++, piximg += n) {
+        double temp = *(pix++) - lin->crpix[j];
+        for (int i = 0; i < naxis; i++, piximg += naxis) {
           img[i] += *piximg * temp;
         }
       }
@@ -782,12 +822,13 @@ int linp2x(
 
   } else {
     // Distortions are present.
-    ndbl = n * sizeof(double);
-    tmp  = lin->tmpcrd;
+    int ndbl = naxis * sizeof(double);
+    register double *tmp  = lin->tmpcrd;
 
-    for (k = 0; k < ncoord; k++) {
+    for (int k = 0; k < ncoord; k++) {
       if (lin->dispre) {
-        if ((status = disp2x(lin->dispre, pix, tmp))) {
+        int status = disp2x(lin->dispre, pix, tmp);
+        if (status) {
           return wcserr_set(LIN_ERRMSG(lin_diserr[status]));
         }
       } else {
@@ -795,37 +836,38 @@ int linp2x(
       }
 
       if (lin->unity) {
-        for (i = 0; i < n; i++) {
+        for (int i = 0; i < naxis; i++) {
           img[i] = tmp[i] - lin->crpix[i];
         }
 
       } else {
-        for (j = 0; j < n; j++) {
+        for (int j = 0; j < naxis; j++) {
           tmp[j] -= lin->crpix[j];
         }
 
-        piximg = lin->piximg;
-        for (i = 0; i < n; i++) {
+        register double *piximg = lin->piximg;
+        for (int i = 0; i < naxis; i++) {
           img[i] = 0.0;
-          for (j = 0; j < n; j++) {
+          for (int j = 0; j < naxis; j++) {
             img[i] += *(piximg++) * tmp[j];
           }
         }
       }
 
       if (lin->disseq) {
-        if ((status = disp2x(lin->disseq, img, tmp))) {
+        int status = disp2x(lin->disseq, img, tmp);
+        if (status) {
           return wcserr_set(LIN_ERRMSG(lin_diserr[status]));
         }
 
         // With sequent distortions, cdelt is not incorporated into piximg...
-        for (i = 0; i < n; i++) {
+        for (int i = 0; i < naxis; i++) {
           img[i] = lin->cdelt[i] * tmp[i];
         }
 
       } else if (lin->unity) {
         // ...nor if the matrix is unity.
-        for (i = 0; i < n; i++) {
+        for (int i = 0; i < naxis; i++) {
           img[i] *= lin->cdelt[i];
         }
       }
@@ -850,32 +892,29 @@ int linx2p(
 {
   static const char *function = "linx2p";
 
-  int i, j, k, n, ndbl, nelemn, status;
-  register const double *img;
-  register double *imgpix, *pix, *tmp;
-  struct wcserr **err;
-
-
   // Initialize.
   if (lin == 0x0) return LINERR_NULL_POINTER;
-  err = &(lin->err);
+  struct wcserr **err = &(lin->err);
 
   if (lin->flag != LINSET) {
-    if ((status = linset(lin))) return status;
+    int status = linset(lin);
+    if (status) {
+      return status;
+    }
   }
 
-  n = lin->naxis;
+  int naxis = lin->naxis;
 
 
   // Convert intermediate world coordinates to pixel coordinates.
-  img = imgcrd;
-  pix = pixcrd;
+  register const double *img = imgcrd;
+  register double *pix = pixcrd;
 
   if (lin->simple) {
     // Handle the simplest and most common case with maximum efficiency.
-    nelemn = nelem - n;
-    for (k = 0; k < ncoord; k++) {
-      for (j = 0; j < n; j++) {
+    int nelemn = nelem - naxis;
+    for (int k = 0; k < ncoord; k++) {
+      for (int j = 0; j < naxis; j++) {
         *(pix++) = (*(img++) / lin->cdelt[j]) + lin->crpix[j];
       }
 
@@ -885,14 +924,14 @@ int linx2p(
 
   } else if (lin->affine) {
     // No distortions.
-    nelemn = nelem - n;
-    for (k = 0; k < ncoord; k++) {
+    int nelemn = nelem - naxis;
+    for (int k = 0; k < ncoord; k++) {
       // cdelt will have been incorporated into imgpix.
-      imgpix = lin->imgpix;
+      register double *imgpix = lin->imgpix;
 
-      for (j = 0; j < n; j++) {
+      for (int j = 0; j < naxis; j++) {
         *pix = 0.0;
-        for (i = 0; i < n; i++) {
+        for (int i = 0; i < naxis; i++) {
           *pix += *imgpix * img[i];
           imgpix++;
         }
@@ -906,17 +945,18 @@ int linx2p(
 
   } else {
     // Distortions are present.
-    ndbl = n * sizeof(double);
-    tmp  = lin->tmpcrd;
+    int ndbl = naxis * sizeof(double);
+    register double *tmp  = lin->tmpcrd;
 
-    for (k = 0; k < ncoord; k++) {
+    for (int k = 0; k < ncoord; k++) {
       if (lin->disseq) {
         // With sequent distortions, cdelt is not incorporated into imgpix...
-        for (i = 0; i < n; i++) {
+        for (int i = 0; i < naxis; i++) {
           tmp[i] = img[i] / lin->cdelt[i];
         }
 
-        if ((status = disx2p(lin->disseq, tmp, pix))) {
+        int status = disx2p(lin->disseq, tmp, pix);
+        if (status) {
           return wcserr_set(LIN_ERRMSG(lin_diserr[status]));
         }
 
@@ -924,7 +964,7 @@ int linx2p(
 
       } else if (lin->unity) {
         // ...nor if the matrix is unity.
-        for (i = 0; i < n; i++) {
+        for (int i = 0; i < naxis; i++) {
           tmp[i] = img[i] / lin->cdelt[i];
         }
 
@@ -934,15 +974,15 @@ int linx2p(
       }
 
       if (lin->unity) {
-        for (j = 0; j < n; j++) {
+        for (int j = 0; j < naxis; j++) {
           pix[j] = tmp[j] + lin->crpix[j];
         }
 
       } else {
-        imgpix = lin->imgpix;
-        for (j = 0; j < n; j++) {
+        register double *imgpix = lin->imgpix;
+        for (int j = 0; j < naxis; j++) {
           pix[j] = lin->crpix[j];
-          for (i = 0; i < n; i++) {
+          for (int i = 0; i < naxis; i++) {
             pix[j] += *(imgpix++) * tmp[i];
           }
         }
@@ -951,7 +991,8 @@ int linx2p(
       if (lin->dispre) {
         memcpy(tmp, pix, ndbl);
 
-        if ((status = disx2p(lin->dispre, tmp, pix))) {
+        int status = disx2p(lin->dispre, tmp, pix);
+        if (status) {
           return wcserr_set(LIN_ERRMSG(lin_diserr[status]));
         }
       }
@@ -982,21 +1023,14 @@ int linwarp(
 {
   static const char *function = "linwarp";
 
-  int carry, i, j, naxis, ncoord, status = 0;
-  double dpix, dpx2, dssq, *img, *pix0, *pix0p, *pix1, *pix1p, *pixend,
-         *pixinc, pixspan, *ssqdis, ssqtot, *sumdis, sumtot, totdis;
-  struct linprm affine;
-  struct wcserr **err;
-
-
   // Initialize.
   if (lin == 0x0) return LINERR_NULL_POINTER;
-  err = &(lin->err);
+  struct wcserr **err = &(lin->err);
 
-  naxis = lin->naxis;
+  int naxis = lin->naxis;
 
   if (nsamp) *nsamp = 0;
-  for (j = 0; j < naxis; j++) {
+  for (int j = 0; j < naxis; j++) {
     if (maxdis) maxdis[j] = 0.0;
     if (avgdis) avgdis[j] = 0.0;
     if (rmsdis) rmsdis[j] = 0.0;
@@ -1010,25 +1044,28 @@ int linwarp(
 
   // It's easier if there are no sequent distortions!
   if (lin->disseq == 0x0) {
-    status = diswarp(lin->dispre, pixblc, pixtrc, pixsamp, nsamp,
-                     maxdis, maxtot, avgdis, avgtot, rmsdis, rmstot);
+    int status = diswarp(lin->dispre, pixblc, pixtrc, pixsamp, nsamp,
+                         maxdis, maxtot, avgdis, avgtot, rmsdis, rmstot);
     return wcserr_set(LIN_ERRMSG(lin_diserr[status]));
   }
 
   // Make a reference copy of lin without distortions.
+  struct linprm affine;
   affine.flag = -1;
-  if ((status = (lincpy(1, lin, &affine) ||
-                 lindist(1, &affine, 0x0, 0) ||
-                 lindist(2, &affine, 0x0, 0) ||
-                 linset(&affine)))) {
+
+  int status = lincpy(1, lin, &affine) ||
+               lindist(1, &affine, 0x0, 0) ||
+               lindist(2, &affine, 0x0, 0) ||
+               linset(&affine);
+  if (status) {
     return wcserr_set(LIN_ERRMSG(status));
   }
 
   // Work out increments on each axis.
-  pixinc = lin->tmpcrd;
-  ncoord = 0;
-  for (j = 0; j < naxis; j++) {
-    pixspan = pixtrc[j] - (pixblc ? pixblc[j] : 1.0);
+  int ncoord = 0;
+  for (int j = 0; j < naxis; j++) {
+    double *pixinc = lin->tmpcrd;
+    double pixspan = pixtrc[j] - (pixblc ? pixblc[j] : 1.0);
 
     if (pixsamp == 0x0) {
       pixinc[j] = 1.0;
@@ -1048,49 +1085,52 @@ int linwarp(
     }
   }
 
-  // Get memory for processing the image row by row.
-  if ((pix0 = calloc((3*ncoord+4)*naxis, sizeof(double))) == 0x0) {
+  // Allocate memory in bulk for processing the image row by row.
+  double *pix0 = calloc((3*ncoord+4)*naxis, sizeof(double));
+  if (pix0 == 0x0) {
     return wcserr_set(LIN_ERRMSG(LINERR_MEMORY));
   }
 
-  img    = pix0 + naxis*ncoord;
-  pix1   = img  + naxis*ncoord;
-  pixinc = pix1 + naxis*ncoord;
-  pixend = pixinc + naxis;
-  sumdis = pixend + naxis;
-  ssqdis = sumdis + naxis;
+  // Carve up the allocated memory.
+  double *img    = pix0 + naxis*ncoord;
+  double *pix1   = img  + naxis*ncoord;
+  double *pixinc = pix1 + naxis*ncoord;
+  double *pixend = pixinc + naxis;
+  double *sumdis = pixend + naxis;
+  double *ssqdis = sumdis + naxis;
 
 
   // Copy tmpcrd since linp2x() will overwrite it.
   memcpy(pixinc, lin->tmpcrd, naxis*sizeof(double));
 
   // Set up the array of pixel coordinates.
-  for (j = 0; j < naxis; j++) {
+  for (int j = 0; j < naxis; j++) {
     pix0[j] = pixblc ? pixblc[j] : 1.0;
     pixend[j] = pixtrc[j] + 0.5*pixinc[j];
   }
 
-  pix0p = pix0 + naxis;
-  for (i = 1; i < ncoord; i++) {
+  double *pix0p = pix0 + naxis;
+  for (int i = 1; i < ncoord; i++) {
     *(pix0p++) = pix0[0] + i*pixinc[0];
 
-    for (j = 1; j < naxis; j++) {
+    for (int j = 1; j < naxis; j++) {
       *(pix0p++) = pix0[j];
     }
   }
 
   // Initialize accumulators.
-  for (j = 0; j < naxis; j++) {
+  for (int j = 0; j < naxis; j++) {
     sumdis[j] = 0.0;
     ssqdis[j] = 0.0;
   }
-  sumtot = 0.0;
-  ssqtot = 0.0;
+  double sumtot = 0.0;
+  double ssqtot = 0.0;
 
 
   // Loop over N dimensions.
-  carry = 0;
+  int carry = 0;
   while (carry == 0) {
+    int status;
     if ((status = linp2x(lin, ncoord, naxis, pix0, img))) {
       // (Preserve the error message set by linp2x().)
       goto cleanup;
@@ -1102,15 +1142,15 @@ int linwarp(
     }
 
     // Accumulate statistics.
-    pix0p = pix0;
-    pix1p = pix1;
-    for (i = 0; i < ncoord; i++) {
+    double *pix0p = pix0;
+    double *pix1p = pix1;
+    for (int i = 0; i < ncoord; i++) {
       (*nsamp)++;
 
-      dssq = 0.0;
-      for (j = 0; j < naxis; j++) {
-        dpix = *(pix1p++) - *(pix0p++);
-        dpx2 = dpix*dpix;
+      double dssq = 0.0;
+      for (int j = 0; j < naxis; j++) {
+        double dpix = *(pix1p++) - *(pix0p++);
+        double dpx2 = dpix*dpix;
 
         sumdis[j] += dpix;
         ssqdis[j] += dpx2;
@@ -1120,7 +1160,7 @@ int linwarp(
         dssq += dpx2;
       }
 
-      totdis = sqrt(dssq);
+      double totdis = sqrt(dssq);
       sumtot += totdis;
       ssqtot += totdis*totdis;
 
@@ -1128,14 +1168,14 @@ int linwarp(
     }
 
     // Next array of pixel coordinates.
-    for (j = 1; j < naxis; j++) {
+    for (int j = 1; j < naxis; j++) {
       pix0[j] += pixinc[j];
       if ((carry = (pix0[j] > pixend[j]))) {
         pix0[j] = pixblc ? pixblc[j] : 1.0;
       }
 
       pix0p = pix0 + naxis + j;
-      for (i = 1; i < ncoord; i++) {
+      for (int i = 1; i < ncoord; i++) {
         *pix0p = pix0[j];
         pix0p += naxis;
       }
@@ -1146,7 +1186,7 @@ int linwarp(
 
 
   // Compute the means and RMSs.
-  for (j = 0; j < naxis; j++) {
+  for (int j = 0; j < naxis; j++) {
     ssqdis[j] /= *nsamp;
     sumdis[j] /= *nsamp;
     if (avgdis) avgdis[j] = sumdis[j];
@@ -1171,27 +1211,27 @@ cleanup:
 int matinv(int n, const double mat[], double inv[])
 
 {
-  register int i, ij, ik, j, k, kj, pj;
-  int    itemp, *mxl, *lxm, pivot;
-  double colmax, *lu, *rowmax, dtemp;
-
-
   // Allocate memory for internal arrays.
-  if ((mxl = calloc(n, sizeof(int))) == 0x0) {
+  int *mxl = calloc(n, sizeof(int));
+  if (mxl == 0x0) {
     return LINERR_MEMORY;
   }
-  if ((lxm = calloc(n, sizeof(int))) == 0x0) {
+
+  int *lxm = calloc(n, sizeof(int));
+  if (lxm == 0x0) {
     free(mxl);
     return LINERR_MEMORY;
   }
 
-  if ((rowmax = calloc(n, sizeof(double))) == 0x0) {
+  double *rowmax = calloc(n, sizeof(double));
+  if (rowmax == 0x0) {
     free(mxl);
     free(lxm);
     return LINERR_MEMORY;
   }
 
-  if ((lu = calloc(n*n, sizeof(double))) == 0x0) {
+  double *lu = calloc(n*n, sizeof(double));
+  if (lu == 0x0) {
     free(mxl);
     free(lxm);
     free(rowmax);
@@ -1200,14 +1240,15 @@ int matinv(int n, const double mat[], double inv[])
 
 
   // Initialize arrays.
-  for (i = 0, ij = 0; i < n; i++) {
+  register int ij = 0;
+  for (int i = 0; i < n; i++) {
     // Vector that records row interchanges.
     mxl[i] = i;
 
     rowmax[i] = 0.0;
 
-    for (j = 0; j < n; j++, ij++) {
-      dtemp = fabs(mat[ij]);
+    for (int j = 0; j < n; j++, ij++) {
+      double dtemp = fabs(mat[ij]);
       if (dtemp > rowmax[i]) rowmax[i] = dtemp;
 
       lu[ij] = mat[ij];
@@ -1225,14 +1266,14 @@ int matinv(int n, const double mat[], double inv[])
 
 
   // Form the LU triangular factorization using scaled partial pivoting.
-  for (k = 0; k < n; k++) {
+  for (int k = 0; k < n; k++) {
     // Decide whether to pivot.
-    colmax = fabs(lu[k*n+k]) / rowmax[k];
-    pivot = k;
+    int pivot = k;
+    double colmax = fabs(lu[k*n+k]) / rowmax[k];
 
-    for (i = k+1; i < n; i++) {
-      ik = i*n + k;
-      dtemp = fabs(lu[ik]) / rowmax[i];
+    for (int i = k+1; i < n; i++) {
+      register int ik = i*n + k;
+      double dtemp = fabs(lu[ik]) / rowmax[i];
       if (dtemp > colmax) {
         colmax = dtemp;
         pivot = i;
@@ -1241,26 +1282,28 @@ int matinv(int n, const double mat[], double inv[])
 
     if (pivot > k) {
       // We must pivot, interchange the rows of the design matrix.
-      for (j = 0, pj = pivot*n, kj = k*n; j < n; j++, pj++, kj++) {
-        dtemp = lu[pj];
+      register int kj = k*n;
+      register int pj = pivot*n;
+      for (int j = 0; j < n; j++, pj++, kj++) {
+        double dtemp = lu[pj];
         lu[pj] = lu[kj];
         lu[kj] = dtemp;
       }
 
       // Amend the vector of row maxima.
-      dtemp = rowmax[pivot];
+      double dtemp = rowmax[pivot];
       rowmax[pivot] = rowmax[k];
       rowmax[k] = dtemp;
 
       // Record the interchange for later use.
-      itemp = mxl[pivot];
+      int itemp = mxl[pivot];
       mxl[pivot] = mxl[k];
       mxl[k] = itemp;
     }
 
     // Gaussian elimination.
-    for (i = k+1; i < n; i++) {
-      ik = i*n + k;
+    for (int i = k+1; i < n; i++) {
+      register int ik = i*n + k;
 
       // Nothing to do if lu[ik] is zero.
       if (lu[ik] != 0.0) {
@@ -1268,7 +1311,7 @@ int matinv(int n, const double mat[], double inv[])
         lu[ik] /= lu[k*n+k];
 
         // Subtract rows.
-        for (j = k+1; j < n; j++) {
+        for (int j = k+1; j < n; j++) {
           lu[i*n+j] -= lu[ik]*lu[k*n+j];
         }
       }
@@ -1278,31 +1321,32 @@ int matinv(int n, const double mat[], double inv[])
 
   // mxl[i] records which row of mat corresponds to row i of lu.
   // lxm[i] records which row of lu  corresponds to row i of mat.
-  for (i = 0; i < n; i++) {
+  for (int i = 0; i < n; i++) {
     lxm[mxl[i]] = i;
   }
 
 
   // Determine the inverse matrix.
-  for (i = 0, ij = 0; i < n; i++) {
-    for (j = 0; j < n; j++, ij++) {
+  ij = 0;
+  for (int i = 0; i < n; i++) {
+    for (int j = 0; j < n; j++, ij++) {
       inv[ij] = 0.0;
     }
   }
 
-  for (k = 0; k < n; k++) {
+  for (int k = 0; k < n; k++) {
     inv[lxm[k]*n+k] = 1.0;
 
     // Forward substitution.
-    for (i = lxm[k]+1; i < n; i++) {
-      for (j = lxm[k]; j < i; j++) {
+    for (int i = lxm[k]+1; i < n; i++) {
+      for (int j = lxm[k]; j < i; j++) {
         inv[i*n+k] -= lu[i*n+j]*inv[j*n+k];
       }
     }
 
     // Backward substitution.
-    for (i = n-1; i >= 0; i--) {
-      for (j = i+1; j < n; j++) {
+    for (int i = n-1; i >= 0; i--) {
+      for (int j = i+1; j < n; j++) {
         inv[i*n+k] -= lu[i*n+j]*inv[j*n+k];
       }
       inv[i*n+k] /= lu[i*n+i];
