@@ -1,5 +1,5 @@
 /*============================================================================
-  WCSLIB 7.4 - an implementation of the FITS WCS standard.
+  WCSLIB 7.7 - an implementation of the FITS WCS standard.
   Copyright (C) 1995-2021, Mark Calabretta
 
   This file is part of WCSLIB.
@@ -19,7 +19,7 @@
 
   Author: Mark Calabretta, Australia Telescope National Facility, CSIRO.
   http://www.atnf.csiro.au/people/Mark.Calabretta
-  $Id: cel_f.c,v 7.4 2021/01/31 02:24:52 mcalabre Exp $
+  $Id: cel_f.c,v 7.7 2021/07/12 06:36:49 mcalabre Exp $
 *===========================================================================*/
 
 #include <stdio.h>
@@ -42,6 +42,7 @@
 
 #define celini_  F77_FUNC(celini,  CELINI)
 #define celfree_ F77_FUNC(celfree, CELFREE)
+#define celsize_ F77_FUNC(celsize, CELSIZE)
 #define celprt_  F77_FUNC(celprt,  CELPRT)
 #define celperr_ F77_FUNC(celperr, CELPERR)
 #define celset_  F77_FUNC(celset,  CELSET)
@@ -228,6 +229,14 @@ int celfree_(int *cel)
 
 //----------------------------------------------------------------------------
 
+int celsize_(const int *cel, int sizes[2])
+
+{
+  return celsize((const struct celprm *)cel, sizes);
+}
+
+//----------------------------------------------------------------------------
+
 int celprt_(const int *cel)
 
 {
@@ -248,9 +257,7 @@ int celperr_(int *cel, const char prefix[72])
 
 {
   char prefix_[73];
-
-  wcsutil_strcvt(72, '\0', prefix, prefix_);
-  prefix_[72] = '\0';
+  wcsutil_strcvt(72, '\0', 1, prefix, prefix_);
 
   // This may or may not force the Fortran I/O buffers to be flushed.
   // If not, try CALL FLUSH(6) before calling CELPERR in the Fortran code.
