@@ -1,5 +1,5 @@
 /*============================================================================
-  WCSLIB 7.4 - an implementation of the FITS WCS standard.
+  WCSLIB 7.7 - an implementation of the FITS WCS standard.
   Copyright (C) 1995-2021, Mark Calabretta
 
   This file is part of WCSLIB.
@@ -19,7 +19,7 @@
 
   Author: Mark Calabretta, Australia Telescope National Facility, CSIRO.
   http://www.atnf.csiro.au/people/Mark.Calabretta
-  $Id: dis_f.c,v 7.4 2021/01/31 02:24:52 mcalabre Exp $
+  $Id: dis_f.c,v 7.7 2021/07/12 06:36:49 mcalabre Exp $
 *=============================================================================
 *
 * In these wrappers, if
@@ -61,6 +61,7 @@
 #define disinit_  F77_FUNC(disinit,  DISINIT)
 #define discpy_   F77_FUNC(discpy,   DISCPY)
 #define disfree_  F77_FUNC(disfree,  DISFREE)
+#define dissize_  F77_FUNC(dissize,  DISSIZE)
 #define disprt_   F77_FUNC(disprt,   DISPRT)
 #define disperr_  F77_FUNC(disperr,  DISPERR)
 #define dishdo_   F77_FUNC(dishdo,   DISHDO)
@@ -149,7 +150,7 @@ int disput_(
     disp->flag = 0;
     break;
   case DIS_DTYPE:
-    wcsutil_strcvt(72, '\0', cvalp, disp->dtype[j0]);
+    wcsutil_strcvt(72, '\0', 0, cvalp, disp->dtype[j0]);
     disp->flag = 0;
     break;
   case DIS_NDP:
@@ -228,7 +229,7 @@ int disget_(const int *deref, const int *dis, const int *what, void *value)
     break;
   case DIS_DTYPE:
     for (j = 0; j < naxis; j++) {
-      wcsutil_strcvt(72, ' ', disp->dtype[j], cvalp);
+      wcsutil_strcvt(72, ' ', 0, disp->dtype[j], cvalp);
       cvalp += 72;
     }
     break;
@@ -354,13 +355,9 @@ int dpfill_(
   double *fval)
 
 {
-  char field_[73], keyword_[73];
-
-  wcsutil_strcvt(72, '\0', keyword, keyword_);
-  keyword_[72] = '\0';
-
-  wcsutil_strcvt(72, '\0', field, field_);
-  field_[72] = '\0';
+  char keyword_[73], field_[73];
+  wcsutil_strcvt(72, '\0', 1, keyword, keyword_);
+  wcsutil_strcvt(72, '\0', 1, field, field_);
 
   return dpfill((struct dpkey *)dp, keyword_, field_, *j, *type, *ival,
                  *fval);
@@ -457,6 +454,14 @@ int disfree_(const int *deref, int *dis)
 
 //----------------------------------------------------------------------------
 
+int dissize_(const int *dis, int sizes[2])
+
+{
+  return dissize((const struct disprm *)dis, sizes);
+}
+
+//----------------------------------------------------------------------------
+
 int disprt_(const int *deref, const int *dis)
 
 {
@@ -484,17 +489,15 @@ int disprt_(const int *deref, const int *dis)
 int disperr_(const int *deref, int *dis, const char prefix[72])
 
 {
-  char prefix_[73];
   const struct disprm *disp;
-
   if (*deref == 0) {
     disp = (const struct disprm *)dis;
   } else {
     disp = *(const struct disprm **)dis;
   }
 
-  wcsutil_strcvt(72, '\0', prefix, prefix_);
-  prefix_[72] = '\0';
+  char prefix_[73];
+  wcsutil_strcvt(72, '\0', 1, prefix, prefix_);
 
   // This may or may not force the Fortran I/O buffers to be flushed.
   // If not, try CALL FLUSH(6) before calling DISPERR in the Fortran code.
