@@ -1,5 +1,5 @@
 /*============================================================================
-  WCSLIB 7.4 - an implementation of the FITS WCS standard.
+  WCSLIB 7.7 - an implementation of the FITS WCS standard.
   Copyright (C) 1995-2021, Mark Calabretta
 
   This file is part of WCSLIB.
@@ -19,7 +19,7 @@
 
   Author: Mark Calabretta, Australia Telescope National Facility, CSIRO.
   http://www.atnf.csiro.au/people/Mark.Calabretta
-  $Id: fitshdr_f.c,v 7.4 2021/01/31 02:24:52 mcalabre Exp $
+  $Id: fitshdr_f.c,v 7.7 2021/07/12 06:36:49 mcalabre Exp $
 *===========================================================================*/
 
 #include <stdio.h>
@@ -73,8 +73,7 @@ int keyidput_(int *keyid, const int *i, const int *what, const void *value)
   switch (*what) {
   case KEYID_NAME:
     // Only eight characters need be given.
-    wcsutil_strcvt(8, ' ', cvalp, kidp->name);
-    kidp->name[8] = '\0';
+    wcsutil_strcvt(8, ' ', 1, cvalp, kidp->name);
     wcsutil_null_fill(12, kidp->name);
     break;
   default:
@@ -105,7 +104,7 @@ int keyidget_(const int *keyid, const int *i, const int *what, void *value)
 
   switch (*what) {
   case KEYID_NAME:
-    wcsutil_strcvt(12, ' ', keyidp->name, cvalp);
+    wcsutil_strcvt(12, ' ', 0, keyidp->name, cvalp);
     break;
   case KEYID_COUNT:
     *ivalp = keyidp->count;
@@ -166,7 +165,7 @@ int keyget_(
     break;
   case KEY_KEYWORD:
     *nc = (int)(strlen(keyp->keyword));
-    wcsutil_strcvt(12, ' ', keyp->keyword, cvalp);
+    wcsutil_strcvt(12, ' ', 0, keyp->keyword, cvalp);
     break;
   case KEY_TYPE:
     *ivalp = keyp->type;
@@ -216,7 +215,7 @@ int keyget_(
     case 8:
       // String or part of a continued string.
       *nc = (int)(strlen(keyp->keyvalue.s));
-      wcsutil_strcvt(72, ' ', keyp->keyvalue.s, cvalp);
+      wcsutil_strcvt(72, ' ', 0, keyp->keyvalue.s, cvalp);
       break;
     default:
       // No value.
@@ -228,7 +227,7 @@ int keyget_(
     break;
   case KEY_COMMENT:
     *nc = (int)(strlen(keyp->comment));
-    wcsutil_strcvt(84, ' ', keyp->comment, cvalp);
+    wcsutil_strcvt(84, ' ', 0, keyp->comment, cvalp);
     break;
   default:
     return 1;
