@@ -1,5 +1,5 @@
 /*============================================================================
-  WCSLIB 7.4 - an implementation of the FITS WCS standard.
+  WCSLIB 7.7 - an implementation of the FITS WCS standard.
   Copyright (C) 1995-2021, Mark Calabretta
 
   This file is part of WCSLIB.
@@ -19,7 +19,7 @@
 
   Author: Mark Calabretta, Australia Telescope National Facility, CSIRO.
   http://www.atnf.csiro.au/people/Mark.Calabretta
-  $Id: lin_f.c,v 7.4 2021/01/31 02:24:52 mcalabre Exp $
+  $Id: lin_f.c,v 7.7 2021/07/12 06:36:49 mcalabre Exp $
 *===========================================================================*/
 
 #include <stdio.h>
@@ -45,6 +45,7 @@
 #define lindist_ F77_FUNC(lindist, LINDIST)
 #define lincpy_  F77_FUNC(lincpy,  LINCPY)
 #define linfree_ F77_FUNC(linfree, LINFREE)
+#define linsize_ F77_FUNC(linsize, LINSIZE)
 #define linprt_  F77_FUNC(linprt,  LINPRT)
 #define linperr_ F77_FUNC(linperr, LINPERR)
 #define linset_  F77_FUNC(linset,  LINSET)
@@ -312,6 +313,14 @@ int linfree_(int *lin)
 
 //----------------------------------------------------------------------------
 
+int linsize_(const int *lin, int sizes[2])
+
+{
+  return linsize((const struct linprm *)lin, sizes);
+}
+
+//----------------------------------------------------------------------------
+
 int linprt_(const int *lin)
 
 {
@@ -332,9 +341,7 @@ int linperr_(int *lin, const char prefix[72])
 
 {
   char prefix_[73];
-
-  wcsutil_strcvt(72, '\0', prefix, prefix_);
-  prefix_[72] = '\0';
+  wcsutil_strcvt(72, '\0', 1, prefix, prefix_);
 
   // This may or may not force the Fortran I/O buffers to be flushed.
   // If not, try CALL FLUSH(6) before calling LINPERR in the Fortran code.
