@@ -1,6 +1,6 @@
 /*============================================================================
-  WCSLIB 7.3 - an implementation of the FITS WCS standard.
-  Copyright (C) 1995-2020, Mark Calabretta
+  WCSLIB 7.7 - an implementation of the FITS WCS standard.
+  Copyright (C) 1995-2021, Mark Calabretta
 
   This file is part of WCSLIB.
 
@@ -17,11 +17,9 @@
   You should have received a copy of the GNU Lesser General Public License
   along with WCSLIB.  If not, see http://www.gnu.org/licenses.
 
-  Direct correspondence concerning WCSLIB to mark@calabretta.id.au
-
   Author: Mark Calabretta, Australia Telescope National Facility, CSIRO.
   http://www.atnf.csiro.au/people/Mark.Calabretta
-  $Id: prj_f.c,v 7.3.1.2 2020/08/17 11:21:34 mcalabre Exp mcalabre $
+  $Id: prj_f.c,v 7.7 2021/07/12 06:36:49 mcalabre Exp $
 *===========================================================================*/
 
 #include <stdio.h>
@@ -44,6 +42,7 @@
 
 #define prjini_  F77_FUNC(prjini,  PRJINI)
 #define prjfree_ F77_FUNC(prjfree, PRJFREE)
+#define prjsize_ F77_FUNC(prjsize, PRJSIZE)
 #define prjprt_  F77_FUNC(prjprt,  PRJPRT)
 #define prjperr_ F77_FUNC(prjperr, PRJPERR)
 #define prjbchk_ F77_FUNC(prjbchk, PRJBCHK)
@@ -93,8 +92,7 @@ int prjput_(int *prj, const int *what, const void *value, const int *m)
     break;
   case PRJ_CODE:
     // Only three characters need be given.
-    wcsutil_strcvt(3, ' ', cvalp, prjp->code);
-    prjp->code[3] = '\0';
+    wcsutil_strcvt(3, ' ', 1, cvalp, prjp->code);
     wcsutil_null_fill(4, prjp->code);
     prjp->flag = 0;
     break;
@@ -163,7 +161,7 @@ int prjget_(const int *prj, const int *what, void *value)
     *ivalp = prjp->flag;
     break;
   case PRJ_CODE:
-    wcsutil_strcvt(4, ' ', prjp->code, cvalp);
+    wcsutil_strcvt(4, ' ', 0, prjp->code, cvalp);
     break;
   case PRJ_R0:
     *dvalp = prjp->r0;
@@ -183,7 +181,7 @@ int prjget_(const int *prj, const int *what, void *value)
     *ivalp = prjp->bounds;
     break;
   case PRJ_NAME:
-    wcsutil_strcvt(40, ' ', prjp->name, cvalp);
+    wcsutil_strcvt(40, ' ', 0, prjp->name, cvalp);
     break;
   case PRJ_CATEGORY:
     *ivalp = prjp->category;
@@ -273,6 +271,14 @@ int prjfree_(int *prj)
 
 //----------------------------------------------------------------------------
 
+int prjsize_(const int *prj, int sizes[2])
+
+{
+  return prjsize((const struct prjprm *)prj, sizes);
+}
+
+//----------------------------------------------------------------------------
+
 int prjprt_(const int *prj)
 
 {
@@ -293,9 +299,7 @@ int prjperr_(int *prj, const char prefix[72])
 
 {
   char prefix_[73];
-
-  wcsutil_strcvt(72, '\0', prefix, prefix_);
-  prefix_[72] = '\0';
+  wcsutil_strcvt(72, '\0', 1, prefix, prefix_);
 
   // This may or may not force the Fortran I/O buffers to be flushed.
   // If not, try CALL FLUSH(6) before calling PRJPERR in the Fortran code.
