@@ -1,6 +1,6 @@
 /*============================================================================
-  WCSLIB 7.3 - an implementation of the FITS WCS standard.
-  Copyright (C) 1995-2020, Mark Calabretta
+  WCSLIB 7.7 - an implementation of the FITS WCS standard.
+  Copyright (C) 1995-2021, Mark Calabretta
 
   This file is part of WCSLIB.
 
@@ -17,11 +17,9 @@
   You should have received a copy of the GNU Lesser General Public License
   along with WCSLIB.  If not, see http://www.gnu.org/licenses.
 
-  Direct correspondence concerning WCSLIB to mark@calabretta.id.au
-
   Author: Mark Calabretta, Australia Telescope National Facility, CSIRO.
   http://www.atnf.csiro.au/people/Mark.Calabretta
-  $Id: spc_f.c,v 7.3.1.2 2020/08/17 11:21:34 mcalabre Exp mcalabre $
+  $Id: spc_f.c,v 7.7 2021/07/12 06:36:49 mcalabre Exp $
 *===========================================================================*/
 
 #include <stdio.h>
@@ -44,6 +42,7 @@
 
 #define spcini_  F77_FUNC(spcini,  SPCINI)
 #define spcfree_ F77_FUNC(spcfree, SPCFREE)
+#define spcsize_ F77_FUNC(spcsize, SPCSIZE)
 #define spcprt_  F77_FUNC(spcprt,  SPCPRT)
 #define spcperr_ F77_FUNC(spcperr, SPCPERR)
 #define spcset_  F77_FUNC(spcset,  SPCSET)
@@ -98,14 +97,12 @@ int spcput_(int *spc, const int *what, const void *value, const int *m)
     break;
   case SPC_TYPE:
     // Only four characters need be given.
-    wcsutil_strcvt(4, ' ', cvalp, spcp->type);
-    spcp->type[4] = '\0';
+    wcsutil_strcvt(4, ' ', 1, cvalp, spcp->type);
     wcsutil_null_fill(8, spcp->type);
     break;
   case SPC_CODE:
     // Only three characters need be given.
-    wcsutil_strcvt(3, ' ', cvalp, spcp->code);
-    spcp->code[3] = '\0';
+    wcsutil_strcvt(3, ' ', 1, cvalp, spcp->code);
     wcsutil_null_fill(4, spcp->code);
     break;
   case SPC_CRVAL:
@@ -166,10 +163,10 @@ int spcget_(const int *spc, const int *what, void *value)
     *ivalp = spcp->flag;
     break;
   case SPC_TYPE:
-    wcsutil_strcvt(8, ' ', spcp->type, cvalp);
+    wcsutil_strcvt(8, ' ', 0, spcp->type, cvalp);
     break;
   case SPC_CODE:
-    wcsutil_strcvt(4, ' ', spcp->code, cvalp);
+    wcsutil_strcvt(4, ' ', 0, spcp->code, cvalp);
     break;
   case SPC_CRVAL:
     *dvalp = spcp->crval;
@@ -246,6 +243,14 @@ int spcfree_(int *spc)
 
 //----------------------------------------------------------------------------
 
+int spcsize_(const int *spc, int sizes[2])
+
+{
+  return spcsize((const struct spcprm *)spc, sizes);
+}
+
+//----------------------------------------------------------------------------
+
 int spcprt_(const int *spc)
 
 {
@@ -266,9 +271,7 @@ int spcperr_(int *spc, const char prefix[72])
 
 {
   char prefix_[73];
-
-  wcsutil_strcvt(72, '\0', prefix, prefix_);
-  prefix_[72] = '\0';
+  wcsutil_strcvt(72, '\0', 1, prefix, prefix_);
 
   // This may or may not force the Fortran I/O buffers to be flushed.
   // If not, try CALL FLUSH(6) before calling SPCPERR in the Fortran code.
@@ -329,19 +332,17 @@ int spctype_(
   iptr err)
 
 {
-  char ctypei_[9], scode_[4], sname_[22], stype_[5], units_[8];
-  int status;
+  char ctypei_[9];
+  wcsutil_strcvt(8, ' ', 1, ctypei, ctypei_);
 
-  wcsutil_strcvt(8, ' ', ctypei, ctypei_);
-  ctypei_[8] = '\0';
+  char stype_[5], scode_[4], sname_[22], units_[8];
+  int status = spctype(ctypei_, stype_, scode_, sname_, units_, ptype, xtype,
+                       restreq, (struct wcserr **)err);
 
-  status = spctype(ctypei_, stype_, scode_, sname_, units_, ptype, xtype,
-                   restreq, (struct wcserr **)err);
-
-  wcsutil_strcvt( 4, ' ', stype_, stype);
-  wcsutil_strcvt( 3, ' ', scode_, scode);
-  wcsutil_strcvt(21, ' ', sname_, sname);
-  wcsutil_strcvt( 7, ' ', units_, units);
+  wcsutil_strcvt( 4, ' ', 0, stype_, stype);
+  wcsutil_strcvt( 3, ' ', 0, scode_, scode);
+  wcsutil_strcvt(21, ' ', 0, sname_, sname);
+  wcsutil_strcvt( 7, ' ', 0, units_, units);
 
   return status;
 }
@@ -379,9 +380,7 @@ int spcspxe_(
 
 {
   char ctypeS_[9];
-
-  wcsutil_strcvt(8, ' ', ctypeS, ctypeS_);
-  ctypeS_[8] = '\0';
+  wcsutil_strcvt(8, ' ', 1, ctypeS, ctypeS_);
 
   return spcspxe(ctypeS_, *crvalS, *restfrq, *restwav, ptype, xtype, restreq,
                  crvalX, dXdS, (struct wcserr **)err);
@@ -421,9 +420,7 @@ int spcxpse_(
 
 {
   char ctypeS_[9];
-
-  wcsutil_strcvt(8, ' ', ctypeS, ctypeS_);
-  ctypeS_[8] = '\0';
+  wcsutil_strcvt(8, ' ', 1, ctypeS, ctypeS_);
 
   return spcxpse(ctypeS_, *crvalX, *restfrq, *restwav, ptype, xtype, restreq,
                  crvalS, dSdX, (struct wcserr **)err);
@@ -461,19 +458,14 @@ int spctrne_(
   iptr err)
 
 {
-  int status;
   char ctypeS1_[9], ctypeS2_[9];
+  wcsutil_strcvt(8, ' ', 1, ctypeS1, ctypeS1_);
+  wcsutil_strcvt(8, ' ', 1, ctypeS2, ctypeS2_);
 
-  wcsutil_strcvt(8, ' ', ctypeS1, ctypeS1_);
-  ctypeS1_[8] = '\0';
+  int status = spctrne(ctypeS1_, *crvalS1, *cdeltS1, *restfrq, *restwav,
+                       ctypeS2_,  crvalS2,  cdeltS2, (struct wcserr **)err);
 
-  wcsutil_strcvt(8, ' ', ctypeS2, ctypeS2_);
-  ctypeS2_[8] = '\0';
-
-  status = spctrne(ctypeS1_, *crvalS1, *cdeltS1, *restfrq, *restwav,
-                   ctypeS2_,  crvalS2,  cdeltS2, (struct wcserr **)err);
-
-  wcsutil_strcvt(8, ' ', ctypeS2_, ctypeS2);
+  wcsutil_strcvt(8, ' ', 0, ctypeS2_, ctypeS2);
 
   return status;
 }
@@ -504,16 +496,14 @@ int spcaips_(
   char specsys[8])
 
 {
-  int status;
-  char ctypeA_[9], ctype_[9], specsys_[9];
+  char ctypeA_[9];
+  wcsutil_strcvt(8, ' ', 1, ctypeA, ctypeA_);
 
-  wcsutil_strcvt(8, ' ', ctypeA, ctypeA_);
-  ctypeA_[8] = '\0';
+  char ctype_[9], specsys_[9];
+  int status = spcaips(ctypeA_, *velref, ctype_, specsys_);
 
-  status = spcaips(ctypeA_, *velref, ctype_, specsys_);
-
-  wcsutil_strcvt(8, ' ', ctype_,   ctype);
-  wcsutil_strcvt(8, ' ', specsys_, specsys);
+  wcsutil_strcvt(8, ' ', 0, ctype_,   ctype);
+  wcsutil_strcvt(8, ' ', 0, specsys_, specsys);
 
   return status;
 }
