@@ -1,6 +1,6 @@
 /*============================================================================
-  WCSLIB 7.3 - an implementation of the FITS WCS standard.
-  Copyright (C) 1995-2020, Mark Calabretta
+  WCSLIB 7.7 - an implementation of the FITS WCS standard.
+  Copyright (C) 1995-2021, Mark Calabretta
 
   This file is part of WCSLIB.
 
@@ -17,11 +17,9 @@
   You should have received a copy of the GNU Lesser General Public License
   along with WCSLIB.  If not, see http://www.gnu.org/licenses.
 
-  Direct correspondence concerning WCSLIB to mark@calabretta.id.au
-
   Author: Mark Calabretta, Australia Telescope National Facility, CSIRO.
   http://www.atnf.csiro.au/people/Mark.Calabretta
-  $Id: spx_f.c,v 7.3.1.2 2020/08/17 11:21:34 mcalabre Exp mcalabre $
+  $Id: spx_f.c,v 7.7 2021/07/12 06:36:49 mcalabre Exp $
 *===========================================================================*/
 
 #include <string.h>
@@ -89,9 +87,7 @@ int specx_(
 
 {
   char type_[5];
-
-  wcsutil_strcvt(4, '\0', type, type_);
-  type_[4] = '\0';
+  wcsutil_strcvt(4, '\0', 1, type, type_);
 
   return specx(type_, *spec, *restfrq, *restwav, (struct spxprm *)specs);
 }
