@@ -1,5 +1,5 @@
 /*============================================================================
-  WCSLIB 7.4 - an implementation of the FITS WCS standard.
+  WCSLIB 7.7 - an implementation of the FITS WCS standard.
   Copyright (C) 1995-2021, Mark Calabretta
 
   This file is part of WCSLIB.
@@ -19,7 +19,7 @@
 
   Author: Mark Calabretta, Australia Telescope National Facility, CSIRO.
   http://www.atnf.csiro.au/people/Mark.Calabretta
-  $Id: tab_f.c,v 7.4 2021/01/31 02:24:52 mcalabre Exp $
+  $Id: tab_f.c,v 7.7 2021/07/12 06:36:49 mcalabre Exp $
 *===========================================================================*/
 
 #include <stdio.h>
@@ -43,6 +43,7 @@
 #define tabcpy_  F77_FUNC(tabcpy,  TABCPY)
 #define tabcmp_  F77_FUNC(tabcmp,  TABCMP)
 #define tabfree_ F77_FUNC(tabfree, TABFREE)
+#define tabsize_ F77_FUNC(tabsize, TABSIZE)
 #define tabprt_  F77_FUNC(tabprt,  TABPRT)
 #define tabperr_ F77_FUNC(tabperr, TABPERR)
 #define tabset_  F77_FUNC(tabset,  TABSET)
@@ -295,6 +296,14 @@ int tabfree_(int *tab)
 
 //----------------------------------------------------------------------------
 
+int tabsize_(const int *tab, int sizes[2])
+
+{
+  return tabsize((const struct tabprm *)tab, sizes);
+}
+
+//----------------------------------------------------------------------------
+
 int tabprt_(const int *tab)
 
 {
@@ -315,9 +324,7 @@ int tabperr_(int *tab, const char prefix[72])
 
 {
   char prefix_[73];
-
-  wcsutil_strcvt(72, '\0', prefix, prefix_);
-  prefix_[72] = '\0';
+  wcsutil_strcvt(72, '\0', 1, prefix, prefix_);
 
   // This may or may not force the Fortran I/O buffers to be flushed.
   // If not, try CALL FLUSH(6) before calling TABPERR in the Fortran code.
