@@ -1,7 +1,7 @@
 *=======================================================================
 *
-* WCSLIB 7.3 - an implementation of the FITS WCS standard.
-* Copyright (C) 1995-2020, Mark Calabretta
+* WCSLIB 7.7 - an implementation of the FITS WCS standard.
+* Copyright (C) 1995-2021, Mark Calabretta
 *
 * This file is part of WCSLIB.
 *
@@ -18,11 +18,9 @@
 * You should have received a copy of the GNU Lesser General Public
 * License along with WCSLIB.  If not, see http://www.gnu.org/licenses.
 *
-* Direct correspondence concerning WCSLIB to mark@calabretta.id.au
-*
 * Author: Mark Calabretta, Australia Telescope National Facility, CSIRO.
 * http://www.atnf.csiro.au/people/Mark.Calabretta
-* $Id: tdis1.f,v 7.3.1.1 2020/06/03 03:38:11 mcalabre Exp mcalabre $
+* $Id: tdis1.f,v 7.7 2021/07/12 06:36:49 mcalabre Exp $
 *=======================================================================
 
       PROGRAM TDIS1
@@ -49,7 +47,7 @@
      :          NWCS, P1, P2, STATUS, WCSP(2)
       DOUBLE PRECISION ABSMAX, CRPIX(AXMAX), DP1, DP2, IMG(2,NMAX), PX,
      :          PX0(2,NMAX), PX1(2,NMAX), RELMAX, RESID
-      CHARACTER KEYREC*80, HEADER*288000, INFILE*9
+      CHARACTER KEYREC*80, HEADER*28801, INFILE*9
 
 *     On some systems, such as Sun Sparc, the structs MUST be aligned
 *     on a double precision boundary, done here using an equivalence.
@@ -58,7 +56,8 @@
       INCLUDE 'wcshdr.inc'
       INCLUDE 'wcs.inc'
       INCLUDE 'lin.inc'
-      INTEGER   WCS(WCSLEN), LIN(LINLEN)
+      INTEGER   WCS(WCSLEN), LIN(LINLEN), ILIN
+      EQUIVALENCE (LIN, ILIN)
       DOUBLE PRECISION DUMMY1, DUMMY2
       EQUIVALENCE (WCS, DUMMY1)
       EQUIVALENCE (LIN, DUMMY2)
@@ -140,7 +139,7 @@
       END IF
 
 *     Henceforth, we will work with linprm.  Make a shallow copy.
-      STATUS = WCSGTI (WCS, WCS_LIN, LIN)
+      STATUS = WCSGTI (WCS, WCS_LIN, ILIN)
       IF (STATUS.NE.0) THEN
         STATUS = WCSPERR (WCS, CHAR(0))
         GO TO 999
