@@ -1,6 +1,6 @@
 *=======================================================================
 *
-* WCSLIB 7.4 - an implementation of the FITS WCS standard.
+* WCSLIB 7.7 - an implementation of the FITS WCS standard.
 * Copyright (C) 1995-2021, Mark Calabretta
 *
 * This file is part of WCSLIB.
@@ -20,7 +20,7 @@
 *
 * Author: Mark Calabretta, Australia Telescope National Facility, CSIRO.
 * http://www.atnf.csiro.au/people/Mark.Calabretta
-* $Id: tdis2.f,v 7.4 2021/01/31 02:24:52 mcalabre Exp $
+* $Id: tdis2.f,v 7.7 2021/07/12 06:36:49 mcalabre Exp $
 *=======================================================================
 
       PROGRAM TDIS2
@@ -44,7 +44,7 @@
       INTEGER   DISP(2), IBLOCK, IKEYRC, J, JK, JN, K, NAXIS, NDP,
      :          NKEYRC, NREJECT, NWCS, STATL, STATP, STATUS, WCSP(2)
       DOUBLE PRECISION MAXDIS(2), TOTDIS
-      CHARACTER DTYPE(2)*72, KEYREC*80, HEADER*288000, INFILE*9
+      CHARACTER DTYPE(2)*72, KEYREC*80, HEADER*28801, INFILE*9
 
 *     On some systems, such as Sun Sparc, the structs MUST be aligned
 *     on a double precision boundary, done here using an equivalence.
@@ -170,9 +170,9 @@
       IF (STATP.NE.0) GO TO 998
       STATP = DISGTI (1, DISP, DIS_NDP,    NDP)
       IF (STATP.NE.0) GO TO 998
-      STATP = DISGTD (1, DISP, DIS_DP,     DP)
+      STATP = DISGTI (1, DISP, DIS_DP,     DP(1))
       IF (STATP.NE.0) GO TO 998
-      STATP = DISGTD (1, DISP, DIS_MAXDIS, MAXDIS)
+      STATP = DISGTD (1, DISP, DIS_MAXDIS, MAXDIS(1))
       IF (STATP.NE.0) GO TO 998
       STATP = DISGTD (1, DISP, DIS_TOTDIS, TOTDIS)
       IF (STATP.NE.0) GO TO 998
@@ -188,7 +188,7 @@
       JK = 0
       JN = 0
       DO 110 J = 1, NAXIS
-        STATL = DISPTD (0, DISL, DIS_DTYPE, DTYPE, J, 0)
+        STATL = DISPTC (0, DISL, DIS_DTYPE, DTYPE, J, 0)
         IF (STATL.NE.0) GO TO 997
  110  CONTINUE
 
@@ -197,11 +197,11 @@
  120  CONTINUE
 
       DO 130 J = 1, NAXIS
-        STATL = DISPTI (0, DISL, DIS_MAXDIS, MAXDIS(J), J, 0)
+        STATL = DISPTD (0, DISL, DIS_MAXDIS, MAXDIS(J), J, 0)
         IF (STATL.NE.0) GO TO 997
  130  CONTINUE
 
-      STATL = DISPTI (0, DISL, DIS_TOTDIS, TOTDIS, J, 0)
+      STATL = DISPTD (0, DISL, DIS_TOTDIS, TOTDIS, J, 0)
       IF (STATL.NE.0) GO TO 997
 
       STATL = DISSET (0, DISL)
