@@ -1,7 +1,7 @@
 *=======================================================================
 *
-* WCSLIB 7.3 - an implementation of the FITS WCS standard.
-* Copyright (C) 1995-2020, Mark Calabretta
+* WCSLIB 7.7 - an implementation of the FITS WCS standard.
+* Copyright (C) 1995-2021, Mark Calabretta
 *
 * This file is part of WCSLIB.
 *
@@ -18,11 +18,9 @@
 * You should have received a copy of the GNU Lesser General Public
 * License along with WCSLIB.  If not, see http://www.gnu.org/licenses.
 *
-* Direct correspondence concerning WCSLIB to mark@calabretta.id.au
-*
 * Author: Mark Calabretta, Australia Telescope National Facility, CSIRO.
 * http://www.atnf.csiro.au/people/Mark.Calabretta
-* $Id: tfitshdr.f,v 7.3.1.1 2020/06/03 03:38:11 mcalabre Exp mcalabre $
+* $Id: tfitshdr.f,v 7.7 2021/07/12 06:36:49 mcalabre Exp $
 *=======================================================================
 
       PROGRAM TFITSHDR
@@ -45,7 +43,7 @@
      :          KTYP, NC, NKEYRC, NKEYID, NREJECT, NWCS, RELAX, STATUS,
      :          ULEN, WCSP(2)
       DOUBLE PRECISION FVAL(2)
-      CHARACTER KEYREC*80, CVAL*72, HEADER*288001, KEYWRD*12, INFILE*12,
+      CHARACTER KEYREC*80, CVAL*72, HEADER*28801, KEYWRD*12, INFILE*12,
      :          TEXT*84
 
       INCLUDE 'wcshdr.inc'
@@ -168,10 +166,10 @@
      :    KEYTYP
 
 *       Format the keyvalue for output.
-        KTYP = MOD(ABS(KEYTYP),10)
+        KTYP = MOD(ABS(KEYTYP), 10)
         IF (KTYP.EQ.1) THEN
 *         Logical.
-          IERR = KEYGTI (KEYS, I, KEY_KEYVALUE, IVAL, NC)
+          IERR = KEYGTI (KEYS, I, KEY_KEYVALUE, IVAL(1), NC)
           IF (IVAL(1).EQ.0) THEN
             TEXT = 'F'
           ELSE
@@ -180,12 +178,12 @@
 
         ELSE IF (KTYP.EQ.2) THEN
 *         32-bit signed integer.
-          IERR = KEYGTI (KEYS, I, KEY_KEYVALUE, IVAL, NC)
+          IERR = KEYGTI (KEYS, I, KEY_KEYVALUE, IVAL(1), NC)
           WRITE (TEXT, '(I11)') IVAL(1)
 
         ELSE IF (KTYP.EQ.3) THEN
 *         64-bit signed integer.
-          IERR = KEYGTI (KEYS, I, KEY_KEYVALUE, IVAL, NC)
+          IERR = KEYGTI (KEYS, I, KEY_KEYVALUE, IVAL(1), NC)
           IF (IVAL(3).NE.0) THEN
             WRITE (TEXT, '(SP,I11,SS,2I9.9)') IVAL(3), ABS(IVAL(2)),
      :             ABS(IVAL(1))
@@ -195,7 +193,7 @@
 
         ELSE IF (KTYP.EQ.4) THEN
 *         Very long integer.
-          IERR = KEYGTI (KEYS, I, KEY_KEYVALUE, IVAL, NC)
+          IERR = KEYGTI (KEYS, I, KEY_KEYVALUE, IVAL(1), NC)
           K = 0
           DO 150 J = 8, 2, -1
             IF (IVAL(J).NE.0) THEN
@@ -276,7 +274,7 @@
       DO 210 I = 0, NKEYID-1
         IERR = KEYIDGTC (KEYIDS, I, KEYID_NAME, TEXT)
         IERR = KEYIDGTI (KEYIDS, I, KEYID_COUNT, NC)
-        IERR = KEYIDGTI (KEYIDS, I, KEYID_IDX, IVAL)
+        IERR = KEYIDGTI (KEYIDS, I, KEYID_IDX, IVAL(1))
         WRITE (*, '(A8,3I5,$)') TEXT, NC, IVAL(1), IVAL(2)
 
 *       Print logical (SIMPLE) and integer (BITPIX, NAXIS) values.
@@ -285,14 +283,14 @@
           WRITE (*, '(I4,$)') KEYTYP
 
           IF (KEYTYP.EQ.1) THEN
-            IERR = KEYGTI (KEYS, I, KEY_KEYVALUE, IVAL, NC)
+            IERR = KEYGTI (KEYS, I, KEY_KEYVALUE, IVAL(1), NC)
             IF (IVAL(1).EQ.0) THEN
               WRITE (*, '(4X,A,$)') 'F'
             ELSE
               WRITE (*, '(4X,A,$)') 'T'
             END IF
           ELSE IF (KEYTYP.EQ.2) THEN
-            IERR = KEYGTI (KEYS, I, KEY_KEYVALUE, IVAL, NC)
+            IERR = KEYGTI (KEYS, I, KEY_KEYVALUE, IVAL(1), NC)
             WRITE (*, '(I5,$)') IVAL(1)
           END IF
         END IF
