@@ -1,5 +1,5 @@
 /*============================================================================
-  WCSLIB 7.4 - an implementation of the FITS WCS standard.
+  WCSLIB 7.7 - an implementation of the FITS WCS standard.
   Copyright (C) 1995-2021, Mark Calabretta
 
   This file is part of WCSLIB.
@@ -19,7 +19,7 @@
 
   Author: Mark Calabretta, Australia Telescope National Facility, CSIRO.
   http://www.atnf.csiro.au/people/Mark.Calabretta
-  $Id: wcs_f.c,v 7.4 2021/01/31 02:24:52 mcalabre Exp $
+  $Id: wcs_f.c,v 7.7 2021/07/12 06:36:49 mcalabre Exp $
 *===========================================================================*/
 
 #include <stdio.h>
@@ -49,6 +49,9 @@
 #define wcssub_  F77_FUNC(wcssub,  WCSSUB)
 #define wcscompare_  F77_FUNC(wcscompare,  WCSCOMPARE)
 #define wcsfree_ F77_FUNC(wcsfree, WCSFREE)
+#define wcstrim_ F77_FUNC(wcstrim, WCSTRIM)
+#define wcssize_ F77_FUNC(wcssize, WCSSIZE)
+#define auxsize_ F77_FUNC(auxsize, AUXSIZE)
 #define wcsprt_  F77_FUNC(wcsprt,  WCSPRT)
 #define wcsperr_ F77_FUNC(wcsperr, WCSPERR)
 #define wcsbchk_ F77_FUNC(wcsbchk, WCSBCHK)
@@ -56,6 +59,7 @@
 #define wcsp2s_  F77_FUNC(wcsp2s,  WCSP2S)
 #define wcss2p_  F77_FUNC(wcss2p,  WCSS2P)
 #define wcsmix_  F77_FUNC(wcsmix,  WCSMIX)
+#define wcsccs_  F77_FUNC(wcsccs,  WCSCCS)
 #define wcssptr_ F77_FUNC(wcssptr, WCSSPTR)
 #define wcscopy_ F77_FUNC(wcscopy, WCSCOPY)
 #define wcslib_version_ F77_FUNC(wcslib_version, WCSLIB_VERSION)
@@ -210,11 +214,11 @@ int wcsput_(
     wcsp->flag = 0;
     break;
   case WCS_CUNIT:
-    wcsutil_strcvt(72, '\0', cvalp, wcsp->cunit[i0]);
+    wcsutil_strcvt(72, '\0', 0, cvalp, wcsp->cunit[i0]);
     wcsp->flag = 0;
     break;
   case WCS_CTYPE:
-    wcsutil_strcvt(72, '\0', cvalp, wcsp->ctype[i0]);
+    wcsutil_strcvt(72, '\0', 0, cvalp, wcsp->ctype[i0]);
     wcsp->flag = 0;
     break;
   case WCS_LONPOLE:
@@ -251,7 +255,7 @@ int wcsput_(
   case WCS_PS:
     (wcsp->ps + wcsp->nps)->i = *i;
     (wcsp->ps + wcsp->nps)->m = *j;
-    wcsutil_strcvt(72, '\0', cvalp, (wcsp->ps + wcsp->nps)->value);
+    wcsutil_strcvt(72, '\0', 0, cvalp, (wcsp->ps + wcsp->nps)->value);
     (wcsp->nps)++;
     wcsp->flag = 0;
     break;
@@ -284,7 +288,7 @@ int wcsput_(
     break;
 
   case WCS_CNAME:
-    wcsutil_strcvt(72, '\0', cvalp, wcsp->cname[i0]);
+    wcsutil_strcvt(72, '\0', 0, cvalp, wcsp->cname[i0]);
     break;
   case WCS_CRDER:
     wcsp->crder[i0] = *dvalp;
@@ -300,26 +304,26 @@ int wcsput_(
     break;
 
   case WCS_WCSNAME:
-    wcsutil_strcvt(72, '\0', cvalp, wcsp->wcsname);
+    wcsutil_strcvt(72, '\0', 0, cvalp, wcsp->wcsname);
     break;
 
   case WCS_TIMESYS:
-    wcsutil_strcvt(72, '\0', cvalp, wcsp->timesys);
+    wcsutil_strcvt(72, '\0', 0, cvalp, wcsp->timesys);
     break;
   case WCS_TREFPOS:
-    wcsutil_strcvt(72, '\0', cvalp, wcsp->trefpos);
+    wcsutil_strcvt(72, '\0', 0, cvalp, wcsp->trefpos);
     break;
   case WCS_TREFDIR:
-    wcsutil_strcvt(72, '\0', cvalp, wcsp->trefdir);
+    wcsutil_strcvt(72, '\0', 0, cvalp, wcsp->trefdir);
     break;
   case WCS_PLEPHEM:
-    wcsutil_strcvt(72, '\0', cvalp, wcsp->plephem);
+    wcsutil_strcvt(72, '\0', 0, cvalp, wcsp->plephem);
     break;
   case WCS_TIMEUNIT:
-    wcsutil_strcvt(72, '\0', cvalp, wcsp->timeunit);
+    wcsutil_strcvt(72, '\0', 0, cvalp, wcsp->timeunit);
     break;
   case WCS_DATEREF:
-    wcsutil_strcvt(72, '\0', cvalp, wcsp->dateref);
+    wcsutil_strcvt(72, '\0', 0, cvalp, wcsp->dateref);
     break;
   case WCS_MJDREF:
     wcsp->mjdref[i0] = *dvalp;
@@ -329,16 +333,16 @@ int wcsput_(
     break;
 
   case WCS_DATEOBS:
-    wcsutil_strcvt(72, '\0', cvalp, wcsp->dateobs);
+    wcsutil_strcvt(72, '\0', 0, cvalp, wcsp->dateobs);
     break;
   case WCS_DATEBEG:
-    wcsutil_strcvt(72, '\0', cvalp, wcsp->datebeg);
+    wcsutil_strcvt(72, '\0', 0, cvalp, wcsp->datebeg);
     break;
   case WCS_DATEAVG:
-    wcsutil_strcvt(72, '\0', cvalp, wcsp->dateavg);
+    wcsutil_strcvt(72, '\0', 0, cvalp, wcsp->dateavg);
     break;
   case WCS_DATEEND:
-    wcsutil_strcvt(72, '\0', cvalp, wcsp->dateend);
+    wcsutil_strcvt(72, '\0', 0, cvalp, wcsp->dateend);
     break;
   case WCS_MJDOBS:
     wcsp->mjdobs = *dvalp;
@@ -388,19 +392,19 @@ int wcsput_(
     wcsp->obsgeo[i0] = *dvalp;
     break;
   case WCS_OBSORBIT:
-    wcsutil_strcvt(72, '\0', cvalp, wcsp->obsorbit);
+    wcsutil_strcvt(72, '\0', 0, cvalp, wcsp->obsorbit);
     break;
   case WCS_RADESYS:
-    wcsutil_strcvt(72, '\0', cvalp, wcsp->radesys);
+    wcsutil_strcvt(72, '\0', 0, cvalp, wcsp->radesys);
     break;
   case WCS_EQUINOX:
     wcsp->equinox = *dvalp;
     break;
   case WCS_SPECSYS:
-    wcsutil_strcvt(72, '\0', cvalp, wcsp->specsys);
+    wcsutil_strcvt(72, '\0', 0, cvalp, wcsp->specsys);
     break;
   case WCS_SSYSOBS:
-    wcsutil_strcvt(72, '\0', cvalp, wcsp->ssysobs);
+    wcsutil_strcvt(72, '\0', 0, cvalp, wcsp->ssysobs);
     break;
   case WCS_VELOSYS:
     wcsp->velosys = *dvalp;
@@ -409,7 +413,7 @@ int wcsput_(
     wcsp->zsource = *dvalp;
     break;
   case WCS_SSYSSRC:
-    wcsutil_strcvt(72, '\0', cvalp, wcsp->ssyssrc);
+    wcsutil_strcvt(72, '\0', 0, cvalp, wcsp->ssyssrc);
     break;
   case WCS_VELANGL:
     wcsp->velangl = *dvalp;
@@ -526,13 +530,13 @@ int wcsget_(const int *wcs, const int *what, void *value)
     break;
   case WCS_CUNIT:
     for (i = 0; i < naxis; i++) {
-      wcsutil_strcvt(72, ' ', wcsp->cunit[i], cvalp);
+      wcsutil_strcvt(72, ' ', 0, wcsp->cunit[i], cvalp);
       cvalp += 72;
     }
     break;
   case WCS_CTYPE:
     for (i = 0; i < naxis; i++) {
-      wcsutil_strcvt(72, ' ', wcsp->ctype[i], cvalp);
+      wcsutil_strcvt(72, ' ', 0, wcsp->ctype[i], cvalp);
       cvalp += 72;
     }
     break;
@@ -572,7 +576,7 @@ int wcsget_(const int *wcs, const int *what, void *value)
       *(dvalp++) = (wcsp->ps + k)->i;
       *(dvalp++) = (wcsp->ps + k)->m;
       cvalp += 2*sizeof(double);
-      wcsutil_strcvt(72, ' ', (wcsp->ps + k)->value, cvalp);
+      wcsutil_strcvt(72, ' ', 0, (wcsp->ps + k)->value, cvalp);
       cvalp += 72;
     }
     break;
@@ -599,7 +603,7 @@ int wcsget_(const int *wcs, const int *what, void *value)
     break;
 
   case WCS_ALT:
-    wcsutil_strcvt(4, ' ', wcsp->alt, cvalp);
+    wcsutil_strcvt(4, ' ', 0, wcsp->alt, cvalp);
     break;
   case WCS_COLNUM:
     *ivalp = wcsp->colnum;
@@ -612,7 +616,7 @@ int wcsget_(const int *wcs, const int *what, void *value)
 
   case WCS_CNAME:
     for (i = 0; i < naxis; i++) {
-      wcsutil_strcvt(72, ' ', wcsp->cname[i], cvalp);
+      wcsutil_strcvt(72, ' ', 0, wcsp->cname[i], cvalp);
       cvalp += 72;
     }
     break;
@@ -638,26 +642,26 @@ int wcsget_(const int *wcs, const int *what, void *value)
     break;
 
   case WCS_WCSNAME:
-    wcsutil_strcvt(72, ' ', wcsp->wcsname, cvalp);
+    wcsutil_strcvt(72, ' ', 0, wcsp->wcsname, cvalp);
     break;
 
   case WCS_TIMESYS:
-    wcsutil_strcvt(72, ' ', wcsp->timesys, cvalp);
+    wcsutil_strcvt(72, ' ', 0, wcsp->timesys, cvalp);
     break;
   case WCS_TREFPOS:
-    wcsutil_strcvt(72, ' ', wcsp->trefpos, cvalp);
+    wcsutil_strcvt(72, ' ', 0, wcsp->trefpos, cvalp);
     break;
   case WCS_TREFDIR:
-    wcsutil_strcvt(72, ' ', wcsp->trefdir, cvalp);
+    wcsutil_strcvt(72, ' ', 0, wcsp->trefdir, cvalp);
     break;
   case WCS_PLEPHEM:
-    wcsutil_strcvt(72, ' ', wcsp->plephem, cvalp);
+    wcsutil_strcvt(72, ' ', 0, wcsp->plephem, cvalp);
     break;
   case WCS_TIMEUNIT:
-    wcsutil_strcvt(72, ' ', wcsp->timeunit, cvalp);
+    wcsutil_strcvt(72, ' ', 0, wcsp->timeunit, cvalp);
     break;
   case WCS_DATEREF:
-    wcsutil_strcvt(72, ' ', wcsp->dateref, cvalp);
+    wcsutil_strcvt(72, ' ', 0, wcsp->dateref, cvalp);
     break;
   case WCS_MJDREF:
     *(dvalp++) = wcsp->mjdref[0];
@@ -668,16 +672,16 @@ int wcsget_(const int *wcs, const int *what, void *value)
     break;
 
   case WCS_DATEOBS:
-    wcsutil_strcvt(72, ' ', wcsp->dateobs, cvalp);
+    wcsutil_strcvt(72, ' ', 0, wcsp->dateobs, cvalp);
     break;
   case WCS_DATEBEG:
-    wcsutil_strcvt(72, ' ', wcsp->datebeg, cvalp);
+    wcsutil_strcvt(72, ' ', 0, wcsp->datebeg, cvalp);
     break;
   case WCS_DATEAVG:
-    wcsutil_strcvt(72, ' ', wcsp->dateavg, cvalp);
+    wcsutil_strcvt(72, ' ', 0, wcsp->dateavg, cvalp);
     break;
   case WCS_DATEEND:
-    wcsutil_strcvt(72, ' ', wcsp->dateend, cvalp);
+    wcsutil_strcvt(72, ' ', 0, wcsp->dateend, cvalp);
     break;
   case WCS_MJDOBS:
     *dvalp = wcsp->mjdobs;
@@ -729,19 +733,19 @@ int wcsget_(const int *wcs, const int *what, void *value)
     }
     break;
   case WCS_OBSORBIT:
-    wcsutil_strcvt(72, ' ', wcsp->obsorbit, cvalp);
+    wcsutil_strcvt(72, ' ', 0, wcsp->obsorbit, cvalp);
     break;
   case WCS_RADESYS:
-    wcsutil_strcvt(72, ' ', wcsp->radesys, cvalp);
+    wcsutil_strcvt(72, ' ', 0, wcsp->radesys, cvalp);
     break;
   case WCS_EQUINOX:
     *dvalp = wcsp->equinox;
     break;
   case WCS_SPECSYS:
-    wcsutil_strcvt(72, ' ', wcsp->specsys, cvalp);
+    wcsutil_strcvt(72, ' ', 0, wcsp->specsys, cvalp);
     break;
   case WCS_SSYSOBS:
-    wcsutil_strcvt(72, ' ', wcsp->ssysobs, cvalp);
+    wcsutil_strcvt(72, ' ', 0, wcsp->ssysobs, cvalp);
     break;
   case WCS_VELOSYS:
     *dvalp = wcsp->velosys;
@@ -750,7 +754,7 @@ int wcsget_(const int *wcs, const int *what, void *value)
     *dvalp = wcsp->zsource;
     break;
   case WCS_SSYSSRC:
-    wcsutil_strcvt(72, ' ', wcsp->ssyssrc, cvalp);
+    wcsutil_strcvt(72, ' ', 0, wcsp->ssyssrc, cvalp);
     break;
   case WCS_VELANGL:
     *dvalp = wcsp->velangl;
@@ -785,10 +789,10 @@ int wcsget_(const int *wcs, const int *what, void *value)
     *(void **)value = wcsp->wtb;
     break;
   case WCS_LNGTYP:
-    wcsutil_strcvt(4, ' ', wcsp->lngtyp, cvalp);
+    wcsutil_strcvt(4, ' ', 0, wcsp->lngtyp, cvalp);
     break;
   case WCS_LATTYP:
-    wcsutil_strcvt(4, ' ', wcsp->lattyp, cvalp);
+    wcsutil_strcvt(4, ' ', 0, wcsp->lattyp, cvalp);
     break;
   case WCS_LNG:
     *ivalp = wcsp->lng + 1;
@@ -894,8 +898,16 @@ int wcsinit_(
 int wcssub_(const int *wcssrc, int *nsub, int axes[], int *wcsdst)
 
 {
-  return wcssub(1, (const struct wcsprm *)wcssrc, nsub, axes,
-                (struct wcsprm *)wcsdst);
+  if (*nsub == -1 && *axes == -1) {
+    // Interpreted as a signal that a deep copy is required.
+    return wcssub(1, (const struct wcsprm *)wcssrc, 0x0, 0x0,
+                  (struct wcsprm *)wcsdst);
+  } else {
+    // Beware that if *nsub == 0 it will be set on return so cannot be a
+    // constant argument.
+    return wcssub(1, (const struct wcsprm *)wcssrc, nsub, axes,
+                  (struct wcsprm *)wcsdst);
+  }
 }
 
 //----------------------------------------------------------------------------
@@ -922,6 +934,30 @@ int wcsfree_(int *wcs)
 
 //----------------------------------------------------------------------------
 
+int wcstrim_(int *wcs)
+
+{
+  return wcstrim((struct wcsprm *)wcs);
+}
+
+//----------------------------------------------------------------------------
+
+int wcssize_(const int *wcs, int sizes[2])
+
+{
+  return wcssize((const struct wcsprm *)wcs, sizes);
+}
+
+//----------------------------------------------------------------------------
+
+int auxsize_(const int *aux, int sizes[2])
+
+{
+  return auxsize((const struct auxprm *)aux, sizes);
+}
+
+//----------------------------------------------------------------------------
+
 int wcsprt_(const int *wcs)
 
 {
@@ -942,9 +978,7 @@ int wcsperr_(int *wcs, const char prefix[72])
 
 {
   char prefix_[73];
-
-  wcsutil_strcvt(72, '\0', prefix, prefix_);
-  prefix_[72] = '\0';
+  wcsutil_strcvt(72, '\0', 1, prefix, prefix_);
 
   // This may or may not force the Fortran I/O buffers to be flushed.
   // If not, try CALL FLUSH(6) before calling WCSPERR in the Fortran code.
@@ -1027,6 +1061,31 @@ int wcsmix_(
 
 //----------------------------------------------------------------------------
 
+int wcsccs_(
+  struct wcsprm *wcs,
+  const double *lng2p1,
+  const double *lat2p1,
+  const double *lng1p2,
+  const char clng[4],
+  const char clat[4],
+  const char radesys[71],
+  const double *equinox,
+  const char alt[1])
+
+{
+  char clng_[5], clat_[5], radesys_[72];
+  wcsutil_strcvt(4,  '\0', 1, clng, clng_);
+  wcsutil_strcvt(4,  '\0', 1, clat, clat_);
+  wcsutil_strcvt(71, '\0', 1, radesys, radesys_);
+
+  int status = wcsccs(wcs, *lng2p1, *lat2p1, *lng1p2, clng_, clat_,
+                      radesys_, *equinox, alt);
+
+  return status;
+}
+
+//----------------------------------------------------------------------------
+
 int wcssptr_(
   struct wcsprm *wcs,
   int *i,
@@ -1034,13 +1093,11 @@ int wcssptr_(
 
 {
   char ctype_[9];
-
-  wcsutil_strcvt(8, ' ', ctype, ctype_);
-  ctype_[8] = '\0';
+  wcsutil_strcvt(8, ' ', 1, ctype, ctype_);
 
   int status = wcssptr(wcs, i, ctype_);
 
-  wcsutil_strcvt(8, ' ', ctype_, ctype);
+  wcsutil_strcvt(8, ' ', 0, ctype_, ctype);
 
   return status;
 }
@@ -1062,5 +1119,5 @@ void wcslib_version_(
   int  nchr)
 
 {
-  wcsutil_strcvt(nchr, ' ', wcslib_version(0x0), wcsver);
+  wcsutil_strcvt(nchr, ' ', 0, wcslib_version(0x0), wcsver);
 }
