@@ -1,5 +1,5 @@
 /*============================================================================
-  WCSLIB 7.4 - an implementation of the FITS WCS standard.
+  WCSLIB 7.7 - an implementation of the FITS WCS standard.
   Copyright (C) 1995-2021, Mark Calabretta
 
   This file is part of WCSLIB.
@@ -19,7 +19,7 @@
 
   Author: Mark Calabretta, Australia Telescope National Facility, CSIRO.
   http://www.atnf.csiro.au/people/Mark.Calabretta
-  $Id: wcsfix_f.c,v 7.4 2021/01/31 02:24:52 mcalabre Exp $
+  $Id: wcsfix_f.c,v 7.7 2021/07/12 06:36:49 mcalabre Exp $
 *===========================================================================*/
 
 #include <wcsfix.h>
@@ -98,3 +98,11 @@ int cylfix_(const int naxis[], int *wcs)
 {
   return cylfix(naxis, (struct wcsprm *)wcs);
 }
+
+//----------------------------------------------------------------------------
+
+int wcspcx_(int *wcs, int *dopc, int *permute, double rotn[2])
+
+{
+  return wcspcx((struct wcsprm *)wcs, *dopc, *permute, rotn);
+}
