@@ -1,6 +1,6 @@
 /*============================================================================
-  PGSBOX 7.3 - draw curvilinear coordinate axes for PGPLOT.
-  Copyright (C) 1997-2020, Mark Calabretta
+  PGSBOX 7.7 - draw curvilinear coordinate axes for PGPLOT.
+  Copyright (C) 1997-2021, Mark Calabretta
 
   This file is part of PGSBOX.
 
@@ -17,11 +17,9 @@
   You should have received a copy of the GNU Lesser General Public License
   along with PGSBOX.  If not, see http://www.gnu.org/licenses.
 
-  Direct correspondence concerning PGSBOX to mark@calabretta.id.au
-
   Author: Mark Calabretta, Australia Telescope National Facility, CSIRO.
   http://www.atnf.csiro.au/people/Mark.Calabretta
-  $Id: cpgtest.c,v 7.3.1.2 2020/08/17 11:24:44 mcalabre Exp mcalabre $
+  $Id: cpgtest.c,v 7.7 2021/07/12 06:36:49 mcalabre Exp $
 *=============================================================================
 *
 *   cpgtest
@@ -50,27 +48,26 @@ int main()
   const double pi = 3.141592653589793238462643;
   const double d2r = pi/180.0;
 
-  char   devtyp[16], fcode[2][4], idents[3][80], nlcprm[1], opt[2];
-  int    c0[7], ci[7], gcode[2], ic, ierr, j, large, naxis[2], nliprm[2];
-  float  blc[2], scl, trc[2];
-  double cache[257][4], dec0, grid1[9], grid2[9], nldprm[8], rotn, tiklen;
-  struct wcsprm wcs;
+  // cpgsbox() function arguments.
+  char   fcode[2][4], idents[3][80], nlcprm[1], opt[2];
+  int    c0[7], ci[7], gcode[2], ic, ierr, nliprm[2];
+  double cache[257][4], grid1[9], grid2[9], nldprm[8];
   nlfunc_t lngvel_, fscan_, pgcrfn_, pgwcsl_;
 
   // Setup.
-  naxis[0] = 512;
-  naxis[1] = 512;
+  int naxis[2] = {512, 512};
 
-  blc[0] = 0.5f;
-  blc[1] = 0.5f;
+  float blc[2] = {0.5f, 0.5f};
+  float trc[2];
   trc[0] = naxis[0] + 0.5f;
   trc[1] = naxis[1] + 0.5f;
 
+  char devtyp[16];
   strcpy(devtyp, "/XWINDOW");
   cpgbeg(0, devtyp, 1, 1);
 
-  j = 16;
-  cpgqinf("TYPE", devtyp, &j);
+  int nchr = 16;
+  cpgqinf("TYPE", devtyp, &nchr);
   if (strcmp(devtyp, "PS") == 0  ||
      strcmp(devtyp, "VPS") == 0 ||
      strcmp(devtyp, "CPS") == 0 ||
@@ -80,7 +77,8 @@ int main()
     cpgscr(1, 0.0f, 0.0f, 0.0f);
   }
 
-  large = strcmp(devtyp, "XWINDOW") == 0;
+  int large = strcmp(devtyp, "XWINDOW") == 0;
+  float scl;
   if (large) {
     scl = 1.0f;
     cpgvstd();
@@ -114,6 +112,7 @@ int main()
   cpgask(1);
   cpgpage();
 
+  struct wcsprm wcs;
   wcs.flag = -1;
 
   //--------------------------------------------------------------------------
@@ -348,7 +347,7 @@ int main()
   wcs.cdelt[1] =  1.0/3600000.0;
 
   // Spherical coordinate references.
-  dec0 = -89.99995;
+  double dec0 = -89.99995;
   wcs.crval[0] = 25.0;
   wcs.crval[1] = dec0;
 
@@ -550,7 +549,7 @@ int main()
   wcs.crpix[1] = 640.0;
 
   // Rotate 30 degrees.
-  rotn = 30.0*d2r;
+  double rotn = 30.0*d2r;
   *(wcs.pc)   =  cos(rotn);
   *(wcs.pc+1) =  sin(rotn);
   *(wcs.pc+2) = -sin(rotn);
@@ -579,7 +578,7 @@ int main()
   // Draw external (TIKLEN < 0) tick marks every 5 degrees.
   gcode[0] = 1;
   gcode[1] = 1;
-  tiklen = -2.0;
+  double tiklen = -2.0;
 
   cpgsci(6);
   grid1[0] = 5.0;
@@ -1198,7 +1197,7 @@ int main()
   // with the two temperature scales, we have the slightly more difficult
   // requirement of labelling grid lines with different values at each
   // end.
-  for (j = 0; j <= ic; j++) {
+  for (int j = 0; j <= ic; j++) {
     // Look for entries associated with the right edge of the frame.
     if (cache[j][0] == 4.0) {
       // Convert to feet, rounding to the nearest 0.1.
