@@ -1,5 +1,5 @@
 /*============================================================================
-  PGSBOX 7.4 - draw curvilinear coordinate axes for PGPLOT.
+  PGSBOX 7.7 - draw curvilinear coordinate axes for PGPLOT.
   Copyright (C) 1997-2021, Mark Calabretta
 
   This file is part of PGSBOX.
@@ -19,7 +19,7 @@
 
   Author: Mark Calabretta, Australia Telescope National Facility, CSIRO.
   http://www.atnf.csiro.au/people/Mark.Calabretta
-  $Id: pgwcsl.c,v 7.4 2021/01/31 02:24:52 mcalabre Exp $
+  $Id: pgwcsl.c,v 7.7 2021/07/12 06:36:49 mcalabre Exp $
 *===========================================================================*/
 
 #include <math.h>
@@ -31,36 +31,35 @@
 #include <wcsconfig_f77.h>
 #define pgwcsl_ F77_FUNC(pgwcsl, PGWCSL)
 
-void pgwcsl_(opcode, nlc, nli, nld, nlcprm, wcs, nldprm, world, pixel, contrl,
-            contxt, ierr)
-
-const int *opcode, *nlc, *nli, *nld;
-const char *nlcprm;
-int *wcs;
-double *nldprm;
-double *world;
-double *pixel;
-int *contrl;
-double contxt[20];
-int *ierr;
+void pgwcsl_(
+  const int *opcode,
+  const int *nlc,
+  const int *nli,
+  const int *nld,
+  const char *nlcprm,
+  int    *wcs,
+  double *nldprm,
+  double *world,
+  double *pixel,
+  int    *contrl,
+  double contxt[20],
+  int    *ierr)
 
 {
-  int i, outside, stat;
-  double dp, imgcrd[9], lat, lng, ph, phi, sdummy, th, theta;
-  static double wrld[9];
-  struct wcsprm *wcsp;
-  struct celprm *wcscel;
-
   // Avert nuisance compiler warnings about unused parameters.
   (void)nlc;
   (void)nld;
   (void)nlcprm;
   (void)nldprm;
 
-  *ierr = 0;
+  int    stat;
+  double imgcrd[9], phi, theta;
+  static double wrld[9];
 
-  wcsp = (struct wcsprm *)wcs;
-  wcscel = &(wcsp->cel);
+  struct wcsprm *wcsp = (struct wcsprm *)wcs;
+  struct celprm *wcscel = &(wcsp->cel);
+
+  *ierr = 0;
 
   if (*opcode == 2) {
     // Compute pixel coordinates from world coordinates.
@@ -73,6 +72,7 @@ int *ierr;
     }
 
     wrld[wcsp->lng] = world[0];
+    int outside;
     if (world[1] > 90.0) {
       wrld[wcsp->lat] = 90.0;
       outside = -2;
@@ -96,6 +96,7 @@ int *ierr;
         contxt[4] = pixel[0];
         contxt[5] = pixel[1];
 
+        double ph, dp;
         if (contxt[2] > phi) {
           ph = 179.9999;
           dp = (phi - contxt[2]) + 360.0;
@@ -105,6 +106,7 @@ int *ierr;
         }
 
         // First approximation for theta.
+        double th;
         if (dp == 0.0) {
           th = contxt[3];
         } else {
@@ -112,6 +114,7 @@ int *ierr;
         }
 
         // Iterate once to refine the value of theta.
+        double lng, lat;
         sphx2s(wcscel->euler, 1, 1, 1, 1, &ph, &th, &lng, &lat);
         if (wrld[wcsp->lng] == contxt[0]) {
           // We are following a meridian of longitude.
@@ -120,6 +123,8 @@ int *ierr;
           // We are following a parallel of latitude.
           lat = wrld[wcsp->lat];
         }
+
+        double sdummy;
         sphs2x(wcscel->euler, 1, 1, 1, 1, &lng, &lat, &sdummy, &th);
 
         contxt[0] = wrld[wcsp->lng];
@@ -181,6 +186,7 @@ int *ierr;
     }
 
     wrld[wcsp->lng] = world[0];
+    int outside;
     if (world[1] > 90.0) {
       wrld[wcsp->lat] = 90.0;
       outside = -2;
@@ -216,7 +222,7 @@ int *ierr;
       *ierr = *ierr <= 2 ? 1 : 2;
     }
 
-    for (i = 2; i < 9; i++) {
+    for (int i = 2; i < 9; i++) {
       wrld[i] = 0.0;
     }
 
