@@ -1,5 +1,5 @@
 /*============================================================================
-  WCSLIB 7.4 - an implementation of the FITS WCS standard.
+  WCSLIB 7.7 - an implementation of the FITS WCS standard.
   Copyright (C) 1995-2021, Mark Calabretta
 
   This file is part of WCSLIB.
@@ -19,12 +19,13 @@
 
   Author: Mark Calabretta, Australia Telescope National Facility, CSIRO.
   http://www.atnf.csiro.au/people/Mark.Calabretta
-  $Id: HPXcvt.c,v 7.4 2021/01/31 02:24:52 mcalabre Exp $
+  $Id: HPXcvt.c,v 7.7 2021/07/12 06:36:49 mcalabre Exp $
 *=============================================================================
-*
+* Usage: HPXcvt [-C<col>] [-c<sys>] [-n|-r] [-q<quad>] [-x<n|s>]
+*               [<infile> [<outfile>]]
+*-----------------------------------------------------------------------------
 * HPXcvt reorganises HEALPix data into a 2-D FITS image.  Refer to the usage
 * notes below.
-*
 *---------------------------------------------------------------------------*/
 
 char usage[] =
@@ -93,9 +94,7 @@ int HPXhdr(fitsfile *fptr, struct healpix *hpxdat);
 int main(int argc, char **argv)
 
 {
-  int crdsys, i, layout, quad, status;
   struct healpix hpxdat;
-
   hpxdat.col      = 1;
   hpxdat.crdsys   = '?';
   hpxdat.ordering = '?';
@@ -103,7 +102,9 @@ int main(int argc, char **argv)
   hpxdat.quad     = 0;
 
   // Parse options.
+  int i;
   for (i = 1; i < argc && argv[i][0] == '-'; i++) {
+    int crdsys, layout, quad;
     if (!argv[i][1]) break;
 
     switch (argv[i][1]) {
@@ -174,8 +175,10 @@ int main(int argc, char **argv)
 
 
   // Get the HEALPix data as a vector.
-  if ((status = HEALPixIn(&hpxdat))) {
-    return 2;
+  int status = HEALPixIn(&hpxdat);
+  if (status) {
+    status = 2;
+    goto cleanup;
   }
 
   if (hpxdat.ordering == '?') {
@@ -187,10 +190,15 @@ int main(int argc, char **argv)
     hpxdat.nside, hpxdat.npix, (hpxdat.ordering == 'N') ? "nested" : "ring");
 
   // Map and write it out as a FITS image.
-  if ((status = HPXout(&hpxdat))) return 3;
+  status = HPXout(&hpxdat);
+  if (status) {
+    status = 3;
+  }
 
+cleanup:
   if (hpxdat.data) free(hpxdat.data);
-  return 0;
+
+  return status;
 }
 
 //----------------------------------------------------------------------------
@@ -198,20 +206,15 @@ int main(int argc, char **argv)
 int HEALPixIn(struct healpix *hpxdat)
 
 {
-  char   crdsys[32], ordering[32];
-  int    anynul, hdutype, iaxis, nfound, status;
-  long   firstpix, ipix, lastpix, naxis, *naxes = 0x0, nside = 0, repeat;
-  float  *datap, nulval;
-  LONGLONG firstelem, irow, nelem = 0, npix = 0, nrow = 0;
-  fitsfile *fptr;
-
-  status = 0;
+  int status = 0;
   hpxdat->data = 0x0;
 
   // Open the FITS file and move to the first HDU with NAXIS != 0.
+  fitsfile *fptr = 0x0;
   if (fits_open_data(&fptr, hpxdat->infile, READONLY, &status)) goto fitserr;
 
   // Is this the primary HDU or an extension?
+  int hdutype;
   if (fits_get_hdu_type(fptr, &hdutype, &status)) goto fitserr;
   if (!(hdutype == IMAGE_HDU || hdutype == BINARY_TBL)) {
     fprintf(stderr, "ERROR: %s does not contain HEALPix data.\n",
@@ -221,14 +224,19 @@ int HEALPixIn(struct healpix *hpxdat)
 
 
   // Get the image size.
+  long naxis;
   if (fits_read_key_lng(fptr, "NAXIS", &naxis, 0x0, &status)) goto fitserr;
+
+  int  nfound;
+  long *naxes = 0x0;
   naxes = malloc(naxis*sizeof(long));
   if (fits_read_keys_lng(fptr, "NAXIS", 1, (int)naxis, naxes, &nfound,
-                         &status)) goto fitserr;
+                         &status)) goto cleanup;
 
+  LONGLONG nelem = 0, npix = 0, nrow = 0;
   if (hdutype == IMAGE_HDU) {
     // Look for the first non-degenerate image axis.
-    for (iaxis = 0; iaxis < nfound; iaxis++) {
+    for (int iaxis = 0; iaxis < nfound; iaxis++) {
       if (naxes[iaxis] > 1) {
         // Assume for now that it is the total number of pixels.
         npix = naxes[iaxis];
@@ -241,8 +249,9 @@ int HEALPixIn(struct healpix *hpxdat)
     if (nfound > 1) nrow = naxes[1];
 
     // (Note that fits_get_coltypell() is not available in cfitsio 2.x.)
+    long repeat;
     if (fits_get_coltype(fptr, hpxdat->col, 0x0, &repeat, 0x0, &status)) {
-      goto fitserr;
+      goto cleanup;
     }
     nelem = (LONGLONG)repeat;
   }
@@ -254,23 +263,24 @@ int HEALPixIn(struct healpix *hpxdat)
 
 
   // Number of pixels per side of each base-resolution pixel.
+  long nside = 0;
   if (fits_read_key_lng(fptr, "NSIDE", &nside, 0x0, &status)) {
     // Some HEALPix files, e.g. SFD dust maps, don't record NSIDE.
-    if (status != KEY_NO_EXIST) goto fitserr;
+    if (status != KEY_NO_EXIST) goto cleanup;
     status = 0;
   }
 
   // FIRSTPIX and LASTPIX, if present, record the 0-relative pixel numbers of
   // the first and last pixels stored in the data.
-  firstpix = -1;
+  long firstpix = -1;
   if (fits_read_key_lng(fptr, "FIRSTPIX", &firstpix, 0x0, &status)) {
-    if (status != KEY_NO_EXIST) goto fitserr;
+    if (status != KEY_NO_EXIST) goto cleanup;
     status = 0;
   }
 
-  lastpix = -1;
+  long lastpix = -1;
   if (fits_read_key_lng(fptr, "LASTPIX", &lastpix, 0x0, &status)) {
-    if (status != KEY_NO_EXIST) goto fitserr;
+    if (status != KEY_NO_EXIST) goto cleanup;
     status = 0;
   }
 
@@ -295,8 +305,9 @@ int HEALPixIn(struct healpix *hpxdat)
 
 
   // Any sign of a coordinate system identifier?
+  char crdsys[32];
   if (fits_read_key_str(fptr, "COORDSYS", crdsys, 0x0, &status)) {
-    if (status != KEY_NO_EXIST) goto fitserr;
+    if (status != KEY_NO_EXIST) goto cleanup;
     status = 0;
   } else if (crdsys[0] == 'G') {
     hpxdat->crdsys = 'G';
@@ -308,9 +319,10 @@ int HEALPixIn(struct healpix *hpxdat)
   }
 
   // Nested or ring ordering?
+  char ordering[32];
   if (fits_read_key_str(fptr, "ORDERING", ordering, 0x0, &status)) {
     // Some HEALPix files, e.g. SFD dust maps, don't record ORDERING.
-    if (status != KEY_NO_EXIST) goto fitserr;
+    if (status != KEY_NO_EXIST) goto cleanup;
     status = 0;
 
   } else if (strcmp(ordering, "NESTED") == 0) {
@@ -325,47 +337,54 @@ int HEALPixIn(struct healpix *hpxdat)
 
 
   // Allocate memory and read the data.
-  if ((hpxdat->data = malloc((hpxdat->npix)*sizeof(float))) == NULL) {
+  if ((hpxdat->data = malloc((hpxdat->npix)*sizeof(float))) == 0x0) {
     perror("HPXcvt");
     goto cleanup;
   }
 
-  nulval = HEALPIX_NULLVAL;
-  datap = hpxdat->data;
-  for (ipix = 0; ipix < firstpix; ipix++) {
+  float nulval = HEALPIX_NULLVAL;
+  float *datap = hpxdat->data;
+  for (long ipix = 0; ipix < firstpix; ipix++) {
     *(datap++) = nulval;
   }
 
-  firstelem = (LONGLONG)1;
+  LONGLONG firstelem = (LONGLONG)1;
   if (hdutype == IMAGE_HDU) {
+    int anynul;
     if (fits_read_img_flt(fptr, 0l, firstelem, npix, nulval, datap, &anynul,
-        &status)) goto fitserr;
+        &status)) goto cleanup;
 
   } else if (hdutype == BINARY_TBL) {
-    for (irow = 0; irow < nrow; irow++) {
+    for (LONGLONG irow = 0; irow < nrow; irow++) {
+      int anynul;
       if (fits_read_col_flt(fptr, hpxdat->col, irow+1, firstelem, nelem,
-          nulval, datap, &anynul, &status)) goto fitserr;
+          nulval, datap, &anynul, &status)) goto cleanup;
       datap += nelem;
     }
   }
 
   datap = hpxdat->data + (lastpix + 1);
-  for (ipix = (lastpix+1); ipix < hpxdat->npix; ipix++) {
+  for (long ipix = (lastpix+1); ipix < hpxdat->npix; ipix++) {
     *(datap++) = nulval;
   }
 
-  // Clean up.
-  fits_close_file(fptr, &status);
-  status = 0;
-  return 0;
-
-fitserr:
-  fits_report_error(stderr, status);
 cleanup:
   if (naxes) free(naxes);
-  if (hpxdat->data) free(hpxdat->data);
-  hpxdat->data = 0x0;
-  return 1;
+
+fitserr:
+  if (status) {
+    if (hpxdat->data) free(hpxdat->data);
+    hpxdat->data = 0x0;
+
+    fits_report_error(stderr, status);
+  }
+
+  if (fptr) {
+    int fstat = 0;
+    fits_close_file(fptr, &fstat);
+  }
+
+  return status?1:0;
 }
 
 //----------------------------------------------------------------------------
@@ -448,23 +467,16 @@ int HPXout(struct healpix *hpxdat)
                                { 0, -2,  3,  0,  0},
                                { 0,  0,  0,  0,  0}}};
 
-  char  history[72];
-  int   facet, halve, i1, i2, ifacet, j, jfacet, layout, nfacet, nside, rotn,
-        status;
-  long  *healidx = 0x0, *healp, naxes[2];
-  float nulval = HEALPIX_NULLVAL, *row = 0x0, *rowp;
-  LONGLONG fpixel, group, nelem;
-  fitsfile *fptr;
-
-
-  nside  = hpxdat->nside;
-  layout = hpxdat->layout;
-  nfacet = NFACET[layout];
+  int nside  = hpxdat->nside;
+  int layout = hpxdat->layout;
+  int nfacet = NFACET[layout];
 
   // Create the output FITS file.
-  status = 0;
+  int  status = 0;
+  long naxes[2];
   naxes[0] = nfacet * nside;
   naxes[1] = naxes[0];
+  fitsfile *fptr;
   if (fits_create_file(&fptr, hpxdat->outfile, &status)) goto fitserr;
   if (fits_create_img(fptr, FLOAT_IMG, 2, naxes, &status)) goto fitserr;
 
@@ -473,24 +485,26 @@ int HPXout(struct healpix *hpxdat)
 
 
   // Allocate arrays.
-  if ((healidx = malloc(nside * sizeof(long)))   == NULL ||
-      (row     = malloc(nside * sizeof(float)))  == NULL) {
+  long  *healidx = 0x0;
+  float *row = 0x0;
+  if ((healidx = malloc(nside * sizeof(long)))   == 0x0 ||
+      (row     = malloc(nside * sizeof(float)))  == 0x0) {
     perror("HPXcvt");
     goto cleanup;
   }
 
   // Loop vertically facet-by-facet.
-  fpixel = 1;
-  group  = 0;
-  nelem  = nside;
-  for (jfacet = 0; jfacet < nfacet; jfacet++) {
+  LONGLONG fpixel = 1;
+  LONGLONG group  = 0;
+  LONGLONG nelem  = nside;
+  for (int jfacet = 0; jfacet < nfacet; jfacet++) {
     // Loop row-by-row.
-    for (j = 0; j < nside; j++) {
+    for (int j = 0; j < nside; j++) {
       // Loop horizontally facet-by-facet.
-      for (ifacet = 0; ifacet < nfacet; ifacet++) {
-        facet = FACETS[layout][jfacet][ifacet];
-        rotn  = FROTAT[layout][jfacet][ifacet];
-        halve = FHALVE[layout][jfacet][ifacet];
+      for (int ifacet = 0; ifacet < nfacet; ifacet++) {
+        int facet = FACETS[layout][jfacet][ifacet];
+        int rotn  = FROTAT[layout][jfacet][ifacet];
+        int halve = FHALVE[layout][jfacet][ifacet];
 
         // Recentre longitude?
         if (hpxdat->quad && facet >= 0) {
@@ -510,7 +524,7 @@ int HPXout(struct healpix *hpxdat)
         if (facet < 0) {
           // A blank facet.
           if (fits_write_img_null(fptr, group, fpixel, nelem, &status)) {
-            goto fitserr;
+            goto cleanup;
           }
 
         } else {
@@ -523,13 +537,15 @@ int HPXout(struct healpix *hpxdat)
           }
 
           // Gather data into the output vector.
-          healp = healidx;
-          for (rowp = row; rowp < row + nside; rowp++) {
+          long *healp = healidx;
+          for (float *rowp = row; rowp < row + nside; rowp++) {
             *rowp = hpxdat->data[*(healp++)];
           }
 
           // Apply blanking to halved facets.
+          float nulval = HEALPIX_NULLVAL;
           if (halve) {
+            int i1, i2;
             if (abs(halve) == 1) {
               // Blank bottom-right.
               i1 = j;
@@ -552,7 +568,7 @@ int HPXout(struct healpix *hpxdat)
               if (halve > 0) i2--;
             }
 
-            for (rowp = row + i1; rowp < row + i2; rowp++) {
+            for (float *rowp = row + i1; rowp < row + i2; rowp++) {
               *rowp = nulval;
             }
           }
@@ -560,7 +576,7 @@ int HPXout(struct healpix *hpxdat)
           // Write out this facet's contribution to this row of the map.
           if (fits_write_imgnull_flt(fptr, group, fpixel, nelem, row, nulval,
                                      &status)) {
-            goto fitserr;
+            goto cleanup;
           }
         }
 
@@ -570,6 +586,7 @@ int HPXout(struct healpix *hpxdat)
   }
 
   // Write history.
+  char history[72];
   sprintf(history, "Original input file: %s", hpxdat->infile);
   fits_write_history(fptr, history, &status);
   sprintf(history, "     Original NSIDE: %d", hpxdat->nside);
@@ -581,15 +598,22 @@ int HPXout(struct healpix *hpxdat)
 
 
   // Clean up.
+  free(healidx);
+  free(row);
   fits_close_file(fptr, &status);
   status = 0;
   return 0;
 
-fitserr:
-  fits_report_error(stderr, status);
 cleanup:
   if (healidx) free(healidx);
   if (row)     free(row);
+fitserr:
+  fits_report_error(stderr, status);
+  if (fptr) {
+    status = 0;
+    fits_close_file(fptr, &status);
+  }
+
   return 1;
 }
 
@@ -603,18 +627,16 @@ cleanup:
 int NESTidx(int nside, int facet, int rotn, int jmap, long *healidx)
 
 {
-  int  h, i, imap, j, nside1, bit;
-  long *hp;
-
   // Nested index (0-relative) of the first pixel in this facet.
-  h = facet * nside * nside;
+  int h = facet * nside * nside;
 
-  nside1 = nside - 1;
-  hp = healidx;
-  for (imap = 0; imap < nside; imap++, hp++) {
+  int  nside1 = nside - 1;
+  long *hp = healidx;
+  for (int imap = 0; imap < nside; imap++, hp++) {
     // (i,j) are 0-relative pixel coordinates with origin in the southern
     // corner of the facet; i increases to the north-east and j to the
     // north-west.
+    int i, j;
     if (rotn == 0) {
       i = nside1 - imap;
       j = jmap;
@@ -630,7 +652,7 @@ int NESTidx(int nside, int facet, int rotn, int jmap, long *healidx)
     }
 
     *hp = 0;
-    bit = 1;
+    int bit = 1;
     while (i || j) {
       if (i & 1) *hp |= bit;
       bit <<= 1;
@@ -658,25 +680,23 @@ int RINGidx(int nside, int facet, int rotn, int jmap, long *healidx)
   const int I0[] = { 1,  3, -3, -1,  0,  2,  4, -2,  1,  3, -3, -1};
   const int J0[] = { 1,  1,  1,  1,  0,  0,  0,  0, -1, -1, -1, -1};
 
-  int  i = 0, i0, imap, j = 0, j0, n2side, n8side, npj, npole, nside1;
-  long *hp;
-
-  n2side = 2 * nside;
-  n8side = 8 * nside;
+  int n2side = 2 * nside;
+  int n8side = 8 * nside;
 
   // Double-pixelisation index of the last pixel in the north polar cap.
-  npole = (n2side - 1) * (n2side - 1) - 1;
+  int npole = (n2side - 1) * (n2side - 1) - 1;
 
   // Double-pixelisation pixel coordinates of the centre of the facet.
-  i0 = nside * I0[facet];
-  j0 = nside * J0[facet];
+  int i0 = nside * I0[facet];
+  int j0 = nside * J0[facet];
 
-  nside1 = nside - 1;
-  hp = healidx;
-  for (imap = 0; imap < nside; imap++, hp++) {
+  int  nside1 = nside - 1;
+  long *hp = healidx;
+  for (int imap = 0; imap < nside; imap++, hp++) {
     // (i,j) are 0-relative, double-pixelisation pixel coordinates.  The
     // origin is at the intersection of the equator and prime meridian,
     // i increases to the east (N.B.) and j to the north.
+    int i = 0, j = 0;
     if (rotn == 0) {
       i = i0 + nside1 - (jmap + imap);
       j = j0 + jmap - imap;
@@ -701,7 +721,7 @@ int RINGidx(int nside, int facet, int rotn, int jmap, long *healidx)
         *hp = 0;
       } else {
         // Number of pixels in a polar facet with this value of j.
-        npj = 2 * (n2side - j);
+        int npj = 2 * (n2side - j);
 
         // Index of the last pixel in the row above this.
         *hp = (npj - 1) * (npj - 1) - 1;
@@ -723,7 +743,7 @@ int RINGidx(int nside, int facet, int rotn, int jmap, long *healidx)
 
       if (j > -n2side) {
         // Number of pixels in a polar facet with this value of j.
-        npj = 2 * (j + n2side);
+        int npj = 2 * (j + n2side);
 
         // Total number of pixels in this row or below it.
         *hp -= (npj + 1) * (npj + 1);
@@ -749,18 +769,14 @@ int RINGidx(int nside, int facet, int rotn, int jmap, long *healidx)
 int HPXhdr(fitsfile *fptr, struct healpix *hpxdat)
 
 {
-  char   comment[64], cval[16], *ctype1, *ctype2, *descr1, *descr2, *pcode;
-  int    status;
-  float  cos45, crpix1, crpix2, crval1, crval2, lonpole;
-  double cdelt1, cdelt2;
-
-  status = 0;
+  int status = 0;
 
   fits_update_key_log(fptr, "EXTEND", 0,
     "No FITS extensions are present", &status);
   fits_write_date(fptr, &status);
 
   // Set pixel transformation parameters.
+  float crpix1, crpix2;
   if (hpxdat->layout == 0) {
     crpix1 = (5 * hpxdat->nside + 1) / 2.0f;
   } else {
@@ -773,7 +789,7 @@ int HPXhdr(fitsfile *fptr, struct healpix *hpxdat)
   fits_write_key(fptr, TFLOAT, "CRPIX2", &crpix2,
     "Coordinate reference pixel", &status);
 
-  cos45 = (float)sqrt(2.0) / 2.0f;
+  float cos45 = (float)sqrt(2.0) / 2.0f;
   if (hpxdat->layout == 0) {
     fits_write_key_flt(fptr, "PC1_1",  cos45, -8,
       "Transformation matrix element", &status);
@@ -785,8 +801,8 @@ int HPXhdr(fitsfile *fptr, struct healpix *hpxdat)
       "Transformation matrix element", &status);
   }
 
-  cdelt1 = -90.0 / hpxdat->nside / sqrt(2.0);
-  cdelt2 = -cdelt1;
+  double cdelt1 = -90.0 / hpxdat->nside / sqrt(2.0);
+  double cdelt2 = -cdelt1;
   fits_write_key_dbl(fptr, "CDELT1", cdelt1, -8,
     "[deg] Coordinate increment", &status);
   fits_write_key_dbl(fptr, "CDELT2", cdelt2, -8,
@@ -794,12 +810,14 @@ int HPXhdr(fitsfile *fptr, struct healpix *hpxdat)
 
 
   // Celestial transformation parameters.
+  char *pcode;
   if (hpxdat->layout == 0) {
     pcode = "HPX";
   } else {
     pcode = "XPH";
   }
 
+  char *ctype1, *ctype2, *descr1, *descr2;
   if (hpxdat->crdsys == 'G') {
     // Galactic.
     ctype1 = "GLON";
@@ -826,7 +844,10 @@ int HPXhdr(fitsfile *fptr, struct healpix *hpxdat)
     descr2 = " Latitude";
   }
 
+  char cval[16];
   sprintf(cval, "%s-%s", ctype1, pcode);
+
+  char comment[64];
   sprintf(comment, "%s in an %s projection", descr1, pcode);
   fits_write_key_str(fptr, "CTYPE1", cval, comment, &status);
   sprintf(cval, "%s-%s", ctype2, pcode);
@@ -834,7 +855,7 @@ int HPXhdr(fitsfile *fptr, struct healpix *hpxdat)
   fits_write_key_str(fptr, "CTYPE2", cval, comment, &status);
 
 
-  crval1 = 0.0f + 90.0f * hpxdat->quad;
+  float crval1 = 0.0f + 90.0f * hpxdat->quad, crval2;
   if (hpxdat->layout == 0) {
     crval2 =   0.0f;
   } else if (hpxdat->layout == 1) {
@@ -852,7 +873,7 @@ int HPXhdr(fitsfile *fptr, struct healpix *hpxdat)
   fits_write_key(fptr, TFLOAT, "CRVAL2", &crval2, comment, &status);
 
   if (hpxdat->layout) {
-    lonpole = 180.0f;
+    float lonpole = 180.0f;
     sprintf(comment, "[deg] Native longitude of the celestial pole");
     fits_write_key(fptr, TFLOAT, "LONPOLE", &lonpole, comment, &status);
   }
